@@ -38,4 +38,5 @@ enum options_client { OPT_CHARSETS_DIR=256, OPT_DEFAULT_CHARSET,
                OPT_SSL_KEY, OPT_SSL_CERT, OPT_SSL_CA, OPT_SSL_CAPATH,
                OPT_SSL_CIPHER, OPT_SHUTDOWN_TIMEOUT, OPT_LOCAL_INFILE,
 	       OPT_DELETE_MASTER_LOGS,
-               OPT_PROMPT, OPT_IGN_LINES,OPT_TRANSACTION, OPT_FRM };
+               OPT_PROMPT, OPT_IGN_LINES,OPT_TRANSACTION, OPT_FRM,
+	       OPT_HEXBLOB, OPT_AUTO_CLOSE };
