@@ -129,6 +129,7 @@ static my_bool info_flag=0,ignore_errors=0,wait_flag=0,quick=0,
 	       vertical=0, line_numbers=1, column_names=1,opt_html=0,
                opt_xml=0,opt_nopager=1, opt_outfile=0, named_cmds= 0,
                tty_password= 0, opt_nobeep=0;
+static ulong opt_max_allowed_packet, opt_net_buffer_length;
 static uint verbose=0,opt_silent=0,opt_mysql_port=0, opt_local_infile=0;
 static my_string opt_mysql_unix_port=0;
 static int connect_flag=CLIENT_INTERACTIVE;
@@ -278,7 +279,8 @@ static void initialize_readline (char *name);
 #endif
 
 static COMMANDS *find_command (char *name,char cmd_name);
-static bool add_line(String &buffer,char *line,char *in_string);
+static bool add_line(String &buffer, char *line, char *in_string, 
+                     my_bool *in_comment);
 static void remove_cntrl(String &buffer);
 static void print_table_data(MYSQL_RES *result);
 static void print_table_data_html(MYSQL_RES *result);
@@ -330,7 +332,7 @@ int main(int argc,char *argv[])
     exit(1);
   }
   if (status.batch && !status.line_buff &&
-      !(status.line_buff=batch_readline_init(max_allowed_packet+512,stdin)))
+      !(status.line_buff=batch_readline_init(opt_max_allowed_packet+512,stdin)))
   {
     free_defaults(defaults_argv);
     exit(1);
@@ -439,6 +441,10 @@ static struct my_option my_long_options[] =
 {
   {"help", '?', "Display this help and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG, 0,
    0, 0, 0, 0, 0},
+#ifdef __NETWARE__
+  {"autoclose", OPT_AUTO_CLOSE, "Auto close the screen on exit for Netware.",
+   0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
+#endif
   {"auto-rehash", OPT_AUTO_REHASH,
    "Enable automatic rehashing. One doesn't need to use 'rehash' to get table and field completion, but startup and reconnecting may take a longer time. Disable with --disable-auto-rehash.",
    (gptr*) &rehash, (gptr*) &rehash, 0, GET_BOOL, NO_ARG, 1, 0, 0, 0, 0, 0},
@@ -573,11 +579,11 @@ static struct my_option my_long_options[] =
    (gptr*) &opt_connect_timeout, 0, GET_ULONG, REQUIRED_ARG, 0, 0, 3600*12, 0,
    0, 1},
   {"max_allowed_packet", OPT_MAX_ALLOWED_PACKET, "",
-   (gptr*) &max_allowed_packet, (gptr*) &max_allowed_packet, 0, GET_ULONG,
+   (gptr*) &opt_max_allowed_packet, (gptr*) &opt_max_allowed_packet, 0, GET_ULONG,
    REQUIRED_ARG, 16 *1024L*1024L, 4096, (longlong) 2*1024L*1024L*1024L,
    MALLOC_OVERHEAD, 1024, 0},
   {"net_buffer_length", OPT_NET_BUFFER_LENGTH, "",
-   (gptr*) &net_buffer_length, (gptr*) &net_buffer_length, 0, GET_ULONG,
+   (gptr*) &opt_net_buffer_length, (gptr*) &opt_net_buffer_length, 0, GET_ULONG,
    REQUIRED_ARG, 16384, 1024, 512*1024*1024L, MALLOC_OVERHEAD, 1024, 0},
   {"select_limit", OPT_SELECT_LIMIT, "", (gptr*) &select_limit,
    (gptr*) &select_limit, 0, GET_ULONG, REQUIRED_ARG, 1000L, 1, ~0L, 0, 1, 0},
@@ -590,6 +596,10 @@ static struct my_option my_long_options[] =
 
 static void usage(int version)
 {
+  /* Divert all help information on NetWare to logger screen. */
+#ifdef __NETWARE__
+#define printf	consoleprintf
+#endif
   printf("%s  Ver %s Distrib %s, for %s (%s)\n",
 	 my_progname, VER, MYSQL_SERVER_VERSION, SYSTEM_TYPE, MACHINE_TYPE);
   if (version)
@@ -602,13 +612,23 @@ and you are welcome to modify and redistribute it under the GPL license\n");
   my_print_help(my_long_options);
   print_defaults("my", load_default_groups);
   my_print_variables(my_long_options);
+  NETWARE_SET_SCREEN_MODE(1);
+#ifdef __NETWARE__
+#undef printf
+#endif
 }
 
+
 static my_bool
 get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
 	       char *argument)
 {
   switch(optid) {
+#ifdef __NETWARE__
+  case OPT_AUTO_CLOSE:
+    setscreenmode(SCR_AUTOCLOSE_ON_EXIT);
+    break;
+#endif
   case OPT_CHARSETS_DIR:
     strmov(mysql_charsets_dir, argument);
     charsets_dir = mysql_charsets_dir;
@@ -678,6 +698,7 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
       while (*argument) *argument++= 'x';		// Destroy argument
       if (*start)
 	start[1]=0 ;
+      tty_password= 0;
     }
     else
       tty_password= 1;
@@ -729,6 +750,7 @@ static int get_options(int argc, char **argv)
 {
   char *tmp, *pagpoint;
   int ho_error;
+  MYSQL_PARAMETERS *mysql_params= mysql_get_parameters();
 
   tmp= (char *) getenv("MYSQL_HOST");
   if (tmp)
@@ -744,9 +766,15 @@ static int get_options(int argc, char **argv)
     strmov(pager, pagpoint);
   strmov(default_pager, pager);
 
+  opt_max_allowed_packet= *mysql_params->p_max_allowed_packet;
+  opt_net_buffer_length= *mysql_params->p_net_buffer_length;
+
   if ((ho_error=handle_options(&argc, &argv, my_long_options, get_one_option)))
     exit(ho_error);
 
+  *mysql_params->p_max_allowed_packet= opt_max_allowed_packet;
+  *mysql_params->p_net_buffer_length= opt_net_buffer_length;
+
   if (status.batch) /* disable pager and outfile in this case */
   {
     strmov(default_pager, "stdout");
@@ -783,6 +811,7 @@ static int read_lines(bool execute_commands)
 #endif
   char	*line;
   char	in_string=0;
+  my_bool in_comment= 0;
   ulong line_number=0;
   COMMANDS *com;
   status.exit_status=1;
@@ -812,6 +841,7 @@ static int read_lines(bool execute_commands)
 #ifdef __NETWARE__
       line=fgets(linebuffer, sizeof(linebuffer)-1, stdin);
       /* Remove the '\n' */
+      if (line)
       {
         char *p = strrchr(line, '\n');
         if (p != NULL)
@@ -827,7 +857,11 @@ static int read_lines(bool execute_commands)
       line= readline(prompt);
 #endif /* defined( __WIN__) || defined(OS2) || defined(__NETWARE__) */
 
-      if (opt_outfile)
+      /*
+        When Ctrl+d or Ctrl+z is pressed, the line may be NULL on some OS
+        which may cause coredump.
+      */
+      if (opt_outfile && line)
 	fprintf(OUTFILE, "%s\n", line);
     }
     if (!line)					// End of file
@@ -857,7 +891,7 @@ static int read_lines(bool execute_commands)
 #endif
       continue;
     }
-    if (add_line(glob_buffer,line,&in_string))
+    if (add_line(glob_buffer, line, &in_string, &in_comment))
       break;
   }
   /* if in batch mode, send last query even if it doesn't end with \g or go */
@@ -917,12 +951,12 @@ static COMMANDS *find_command (char *name,char cmd_char)
 }
 
 
-static bool add_line(String &buffer,char *line,char *in_string)
+static bool add_line(String &buffer,char *line,char *in_string, 
+                     my_bool *in_comment)
 {
   uchar inchar;
   char buff[80],*pos,*out;
   COMMANDS *com;
-  my_bool in_comment= 0;
 
   if (!line[0] && buffer.is_empty())
     return 0;
@@ -982,7 +1016,7 @@ static bool add_line(String &buffer,char *line,char *in_string)
 	continue;
       }
     }
-    else if (inchar == ';' && !*in_string && !in_comment)
+    else if (inchar == ';' && !*in_string && !*in_comment)
     {						// ';' is end of command
       if (out != line)
 	buffer.append(line,(uint) (out-line));	// Add this line
@@ -1010,15 +1044,15 @@ static bool add_line(String &buffer,char *line,char *in_string)
     {						// Add found char to buffer
       if (inchar == *in_string)
 	*in_string=0;
-      else if (!in_comment && !*in_string && (inchar == '\'' || inchar == '"' || inchar == '`'))
+      else if (!*in_comment && !*in_string && (inchar == '\'' || inchar == '"' || inchar == '`'))
 	*in_string=(char) inchar;
       *out++ = (char) inchar;
       if (inchar == '*' && !*in_string)
       {
 	if (pos != line && pos[-1] == '/')
-	  in_comment= 1;
+	  *in_comment= 1;
 	else if (in_comment && pos[1] == '/')
-	  in_comment= 0;
+	  *in_comment= 0;
       }
     }
   }
@@ -2018,10 +2052,8 @@ static int
 com_quit(String *buffer __attribute__((unused)),
 	 char *line __attribute__((unused)))
 {
-#ifdef __NETWARE__
-  // let the screen auto close on a normal shutdown
-  setscreenmode(SCR_AUTOCLOSE_ON_EXIT);
-#endif
+  /* let the screen auto close on a normal shutdown */
+  NETWARE_SET_SCREEN_MODE(SCR_AUTOCLOSE_ON_EXIT);
   status.exit_status=0;
   return 1;
 }
@@ -2152,7 +2184,7 @@ static int com_source(String *buffer, char *line)
     return put_info(buff, INFO_ERROR, 0);
   }
 
-  if (!(line_buff=batch_readline_init(max_allowed_packet+512,sql_file)))
+  if (!(line_buff=batch_readline_init(opt_max_allowed_packet+512,sql_file)))
   {
     my_fclose(sql_file,MYF(0));
     return put_info("Can't initialize batch_readline", INFO_ERROR, 0);
@@ -2178,20 +2210,77 @@ static int com_source(String *buffer, char *line)
 static int
 com_use(String *buffer __attribute__((unused)), char *line)
 {
-  char *tmp;
-  char buff[256];
+  char tmp[FN_REFLEN], buff[FN_REFLEN + 1];
+  MYSQL_RES *res;
+  MYSQL_ROW row;
+  char *c_buff, *c_tmp;
 
   while (isspace(*line))
     line++;
   strnmov(buff,line,sizeof(buff)-1);		// Don't destroy history
   if (buff[0] == '\\')				// Short command
     buff[1]=' ';
-  tmp=(char *) strtok(buff," \t;");		// Skip connect command
-  if (!tmp || !(tmp=(char *) strtok(NullS," \t;")))
+  c_buff= buff;
+  while ((*c_buff != ' ') && (*c_buff != '\t')) // Skip connect command
+    c_buff++;
+  c_buff++;
+
+  while ((*c_buff == ' ') || (*c_buff == '\t'))
+    c_buff++;
+  c_tmp= tmp;
+  if (*c_buff == '`')                           // Handling backticks
+  {
+    c_buff++;
+    for (; *c_buff; c_tmp++)
+    {
+      if (*c_buff == '`')
+      {
+	if (c_buff[1] == '`')
+	{
+	  *c_tmp= '`';
+	  c_buff+= 2;
+	}
+	else
+	  break;
+      }
+      else
+	*c_tmp= *(c_buff++);
+    }
+  }
+  else
+    for (; !strchr(" \t;", *c_buff); c_buff++, c_tmp++)
+      *c_tmp= *c_buff;
+  *c_tmp= '\0';
+  
+  if (!*tmp)
   {
     put_info("USE must be followed by a database name",INFO_ERROR);
     return 0;
   }
+  /*
+    We need to recheck the current database, because it may change
+    under our feet, for example if DROP DATABASE or RENAME DATABASE
+    (latter one not yet available by the time the comment was written)
+  */
+  /*  Let's reset current_db, assume it's gone */
+  my_free(current_db, MYF(MY_ALLOW_ZERO_PTR));
+  current_db= 0;
+  /*
+    We don't care about in case of an error below because current_db
+    was just set to 0.
+  */
+  if (!mysql_query(&mysql, "SELECT DATABASE()") &&
+      (res= mysql_use_result(&mysql)))
+  {
+    row= mysql_fetch_row(res);
+    if (row[0])
+    {
+      current_db= my_strdup(row[0], MYF(MY_WME));
+    }
+    (void) mysql_fetch_row(res);               // Read eof
+    mysql_free_result(res);
+  }
+
   if (!current_db || cmp_database(current_db,tmp))
   {
     if (one_database)
@@ -2331,13 +2420,16 @@ com_status(String *buffer __attribute__((unused)),
     MYSQL_RES *result;
     LINT_INIT(result);
     tee_fprintf(stdout, "\nConnection id:\t\t%lu\n",mysql_thread_id(&mysql));
-    if (!mysql_query(&mysql,"select DATABASE(),USER()") &&
+    if (!mysql_query(&mysql,"select DATABASE(), USER() limit 1") &&
 	(result=mysql_use_result(&mysql)))
     {
       MYSQL_ROW cur=mysql_fetch_row(result);
-      tee_fprintf(stdout, "Current database:\t%s\n", cur[0] ? cur[0] : "");
-      tee_fprintf(stdout, "Current user:\t\t%s\n",cur[1]);
-      (void) mysql_fetch_row(result);		// Read eof
+      if (cur)
+      {
+        tee_fprintf(stdout, "Current database:\t%s\n", cur[0] ? cur[0] : "");
+        tee_fprintf(stdout, "Current user:\t\t%s\n", cur[1]);
+      }
+      mysql_free_result(result);
     }
 #ifdef HAVE_OPENSSL
     if (mysql.net.vio && mysql.net.vio->ssl_arg &&
@@ -2489,6 +2581,7 @@ void tee_fprintf(FILE *file, const char *fmt, ...)
 {
   va_list args;
 
+  NETWARE_YIELD;
   va_start(args, fmt);
   (void) vfprintf(file, fmt, args);
 #ifdef OS2
@@ -2502,6 +2595,7 @@ void tee_fprintf(FILE *file, const char *fmt, ...)
 
 void tee_fputs(const char *s, FILE *file)
 {
+  NETWARE_YIELD;
   fputs(s, file);
 #ifdef OS2
   fflush( file);
@@ -2513,6 +2607,7 @@ void tee_fputs(const char *s, FILE *file)
 
 void tee_puts(const char *s, FILE *file)
 {
+  NETWARE_YIELD;
   fputs(s, file);
   fputs("\n", file);
 #ifdef OS2
