@@ -106,6 +106,10 @@ static TYPELIB command_typelib=
 
 static struct my_option my_long_options[] =
 {
+#ifdef __NETWARE__
+  {"autoclose", OPT_AUTO_CLOSE, "Auto close the screen on exit for Netware.",
+   0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
+#endif
   {"count", 'c',
    "Number of iterations to make. This works with -i (--sleep) only",
    (gptr*) &nr_iterations, (gptr*) &nr_iterations, 0, GET_UINT,
@@ -184,6 +188,11 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
   int error = 0;
 
   switch(optid) {
+#ifdef __NETWARE__
+  case OPT_AUTO_CLOSE:
+    setscreenmode(SCR_AUTOCLOSE_ON_EXIT);
+    break;
+#endif
   case 'c':
     opt_count_iterations= 1;
     break;
@@ -246,7 +255,7 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
 
 int main(int argc,char *argv[])
 {
-  int error, ho_error;
+  int error= 0, ho_error;
   MYSQL mysql;
   char **commands, **save_argv;
 
@@ -285,10 +294,25 @@ int main(int argc,char *argv[])
 		  opt_ssl_capath, opt_ssl_cipher);
 #endif
   if (sql_connect(&mysql, option_wait))
-    error = 1;
+  {
+    unsigned int err= mysql_errno(&mysql);
+    if (err >= CR_MIN_ERROR && err <= CR_MAX_ERROR)
+      error= 1;
+    else
+    {
+      /* Return 0 if all commands are PING */
+      for (; argc > 0; argv++, argc--)
+      {
+        if (find_type(argv[0], &command_typelib, 2) != ADMIN_PING)
+        {
+          error= 1;
+          break;
+        }
+      }
+    }
+  }
   else
   {
-    error = 0;
     while (!interrupted && (!opt_count_iterations || nr_iterations))
     {
       new_line = 0;
@@ -539,7 +563,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
       {
 	char *pos,buff[40];
 	ulong sec;
-	pos=strchr(status,' ');
+	pos= (char*) strchr(status,' ');
 	*pos++=0;
 	printf("%s\t\t\t",status);			/* print label */
 	if ((status=str2int(pos,10,0,LONG_MAX,(long*) &sec)))
@@ -815,11 +839,13 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
   return 0;
 }
 
+#include <help_start.h>
 
 static void print_version(void)
 {
   printf("%s  Ver %s Distrib %s, for %s on %s\n",my_progname,ADMIN_VERSION,
 	 MYSQL_SERVER_VERSION,SYSTEM_TYPE,MACHINE_TYPE);
+  NETWARE_SET_SCREEN_MODE(1);
 }
 
 
@@ -861,6 +887,8 @@ static void usage(void)
   version		Get version info from server");
 }
 
+#include <help_end.h>
+
 static int drop_db(MYSQL *mysql, const char *db)
 {
   char name_buff[FN_REFLEN+20], buf[10];
