@@ -400,6 +400,11 @@ Create_file event for file_id: %u\n",exv->file_id);
 
 static struct my_option my_long_options[] =
 {
+
+#ifdef __NETWARE__
+  {"autoclose", OPT_AUTO_CLOSE, "Auto close the screen on exit for Netware.",
+   0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
+#endif
 #ifndef DBUG_OFF
   {"debug", '#', "Output debug log.", (gptr*) &default_dbug_option,
    (gptr*) &default_dbug_option, 0, GET_STR, OPT_ARG, 0, 0, 0, 0, 0, 0},
@@ -479,9 +484,12 @@ static void die(const char* fmt, ...)
   exit(1);
 }
 
+#include <help_start.h>
+
 static void print_version()
 {
   printf("%s Ver 2.6 for %s at %s\n", my_progname, SYSTEM_TYPE, MACHINE_TYPE);
+  NETWARE_SET_SCREEN_MODE(1);
 }
 
 
@@ -500,12 +508,19 @@ the mysql command line client\n\n");
   my_print_variables(my_long_options);
 }
 
+#include <help_end.h>
+
 extern "C" my_bool
 get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
 	       char *argument)
 {
   bool tty_password=0;
   switch (optid) {
+#ifdef __NETWARE__
+  case OPT_AUTO_CLOSE:
+    setscreenmode(SCR_AUTOCLOSE_ON_EXIT);
+    break;
+#endif
 #ifndef DBUG_OFF
   case '#':
     DBUG_PUSH(argument ? argument : default_dbug_option);
@@ -718,8 +733,8 @@ static int dump_remote_log_entries(const char* logname)
     */
     if (old_off)
       old_off+= len-1;
-    else
-      old_off= BIN_LOG_HEADER_SIZE;
+    else                        // first event, so it's a fake Rotate event
+      old_off= position;
   }
   return 0;
 }
@@ -852,7 +867,7 @@ typedef struct st_my_tmpdir
   uint cur, max;
 } MY_TMPDIR;
 
-#if defined( __WIN__) || defined(OS2)
+#if defined( __WIN__) || defined(OS2) || defined(__NETWARE__)
 #define DELIM ';'
 #else
 #define DELIM ':'
