@@ -56,6 +56,10 @@ static struct my_option my_long_options[] =
    "Instead of issuing one query for each table, use one query per database, naming all tables in the database in a comma-separated list.",
    (gptr*) &opt_all_in_1, (gptr*) &opt_all_in_1, 0, GET_BOOL, NO_ARG, 0, 0, 0,
    0, 0, 0},
+#ifdef __NETWARE__
+  {"autoclose", OPT_AUTO_CLOSE, "Auto close the screen on exit for Netware.",
+   0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
+#endif
   {"auto-repair", OPT_AUTO_REPAIR,
    "If a checked table is corrupted, automatically fix it. Repairing will be done after all tables have been checked, if corrupted ones were found.",
    (gptr*) &opt_auto_repair, (gptr*) &opt_auto_repair, 0, GET_BOOL, NO_ARG, 0,
@@ -159,10 +163,13 @@ static void print_result();
 static char *fix_table_name(char *dest, char *src);
 int what_to_do = 0;
 
+#include <help_start.h>
+
 static void print_version(void)
 {
   printf("%s  Ver %s Distrib %s, for %s (%s)\n", my_progname, CHECK_VERSION,
    MYSQL_SERVER_VERSION, SYSTEM_TYPE, MACHINE_TYPE);
+  NETWARE_SET_SCREEN_MODE(1);
 } /* print_version */
 
 
@@ -193,12 +200,18 @@ static void usage(void)
   my_print_variables(my_long_options);
 } /* usage */
 
+#include <help_end.h>
 
 static my_bool
 get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
 	       char *argument)
 {
   switch(optid) {
+#ifdef __NETWARE__
+  case OPT_AUTO_CLOSE:
+    setscreenmode(SCR_AUTOCLOSE_ON_EXIT);
+    break;
+#endif
   case 'a':
     what_to_do = DO_ANALYZE;
     break;
@@ -409,18 +422,18 @@ static int process_all_tables_in_db(char *database)
   LINT_INIT(res);
   if (use_db(database))
     return 1;
-  if (!(mysql_query(sock, "SHOW TABLES") ||
-	(res = mysql_store_result(sock))))
+  if (mysql_query(sock, "SHOW TABLES") ||
+	!((res= mysql_store_result(sock))))
     return 1;
 
   if (opt_all_in_1)
   {
-    /* 
+    /*
       We need table list in form `a`, `b`, `c`
       that's why we need 4 more chars added to to each table name
       space is for more readable output in logs and in case of error
      */
-	  
+
     char *tables, *end;
     uint tot_length = 0;
 
@@ -533,6 +546,7 @@ static void print_result()
   my_bool found_error=0;
 
   res = mysql_use_result(sock);
+
   prev[0] = '\0';
   for (i = 0; (row = mysql_fetch_row(res)); i++)
   {
@@ -561,7 +575,7 @@ static void print_result()
     putchar('\n');
   }
   if (found_error && opt_auto_repair && what_to_do != DO_REPAIR &&
-      (!opt_fast || strcmp(row[3],"OK")))
+      !opt_fast)
     insert_dynamic(&tables4repair, prev);
   mysql_free_result(res);
 }
