@@ -16,7 +16,7 @@
 
 /* Return error-text for system error messages and nisam messages */
 
-#define PERROR_VERSION "2.9"
+#define PERROR_VERSION "2.10"
 
 #include <my_global.h>
 #include <my_sys.h>
@@ -101,12 +101,15 @@ static HA_ERRORS ha_errlist[]=
 };
 
 
+#include <help_start.h>
+
 static void print_version(void)
 {
   printf("%s Ver %s, for %s (%s)\n",my_progname,PERROR_VERSION,
 	 SYSTEM_TYPE,MACHINE_TYPE);
 }
 
+
 static void usage(void)
 {
   print_version();
@@ -118,6 +121,8 @@ static void usage(void)
   my_print_variables(my_long_options);
 }
 
+#include <help_end.h>
+
 
 static my_bool
 get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
@@ -195,16 +200,30 @@ int main(int argc,char *argv[])
   else
 #endif
   {
+    /*
+      On some system, like NETWARE, strerror(unknown_error) returns a
+      string 'Unknown Error'.  To avoid printing it we try to find the
+      error string by asking for an impossible big error message.
+    */
+    const char *unknown_error= strerror(10000);
+
     for ( ; argc-- > 0 ; argv++)
     {
       found=0;
       code=atoi(*argv);
-      msg = strerror(code);
-      if (msg)
+      msg= strerror(code);
+
+      /*
+        Don't print message for not existing error messages or for
+        unknown errors.  We test for 'Uknown Errors' just as an
+        extra safety for Netware
+      */
+      if (msg && strcmp(msg, "Unknown Error") &&
+          (!unknown_error || strcmp(msg, unknown_error)))
       {
 	found=1;
 	if (verbose)
-	  printf("Error code %3d:  %s\n",code,msg);
+	  printf("System error: %3d = %s\n",code,msg);
 	else
 	  puts(msg);
       }
@@ -219,7 +238,7 @@ int main(int argc,char *argv[])
       else
       {
 	if (verbose)
-	  printf("%3d = %s\n",code,msg);
+	  printf("MySQL error:  %3d = %s\n",code,msg);
 	else
 	  puts(msg);
       }
