@@ -14,12 +14,16 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
-/* Replace strings in textfile
-  This program replace strings in a file or on stdin/stdout.
-  It accepts a list of from-strings and to-strings and replaces all
-  occurents of from-strings to to-strings.
-  The first occurents of a found string is matched. If there are more than
-  one possibly replace the longer from-string is replaced.
+/*
+  Replace strings in textfile
+
+  This program replaces strings in files or from stdin to stdout.
+  It accepts a list of from-string/to-string pairs and replaces
+  each occurrence of a from-string with the corresponding to-string.
+  The first occurrence of a found string is matched. If there is more
+  than one possibility for the string to replace, longer matches
+  are preferred before shorter matches.
+
   Special characters in from string:
   \^    Match start of line.
   \$	Match end of line.
@@ -167,7 +171,7 @@ register char **argv[];
 	break;
       case '#':
 	DBUG_PUSH (++pos);
-	pos= (char*) " ";			/* Skipp rest of arguments */
+	pos= (char*) " ";			/* Skip rest of arguments */
 	break;
       case 'V':
 	version=1;
@@ -179,12 +183,13 @@ register char **argv[];
 	if (version)
 	  break;
 	puts("This software comes with ABSOLUTELY NO WARRANTY. This is free software,\nand you are welcome to modify and redistribute it under the GPL license\n");
-	puts("This program replace strings in a file or on stdin/stdout.\n"
-	     "It accepts a list of from-strings and to-strings and replaces\n"
-	     "all occurents of from-strings to to-strings.\n"
-	     "The first occurents of a found string is matched. Longer matches\n"
-	     "are prefered before shorter matches.\n\n"
-	     "Special characters in from string:\n"
+	puts("This program replaces strings in files or from stdin to stdout.\n"
+	     "It accepts a list of from-string/to-string pairs and replaces\n"
+	     "each occurrence of a from-string with the corresponding to-string.\n"
+         "The first occurrence of a found string is matched. If there is\n"
+         "more than one possibility for the string to replace, longer\n"
+         "matches are preferred before shorter matches.\n\n"
+	     "A from-string can contain these special characters:\n"
 	     "  \\^      Match start of line.\n"
 	     "  \\$      Match end of line.\n"
 	     "  \\b      Match space-character, start of line or end of line.\n"
@@ -237,7 +242,7 @@ POINTER_ARRAY *from_array,*to_array;
     (*argv)++;
   }
   if (*argc)
-  {					/* Skipp "--" argument */
+  {					/* Skip "--" argument */
     (*argc)--;
     (*argv)++;
   }
@@ -274,8 +279,10 @@ int insert_pointer_name(reg1 POINTER_ARRAY *pa,my_string name)
   length=(uint) strlen(name)+1;
   if (pa->length+length >= pa->max_length)
   {
+    pa->max_length=(pa->length+length+MALLOC_OVERHEAD+PS_MALLOC-1)/PS_MALLOC;
+    pa->max_length=pa->max_length*PS_MALLOC-MALLOC_OVERHEAD;
     if (!(new_pos= (byte*) my_realloc((gptr) pa->str,
-				      (uint) (pa->max_length+PS_MALLOC),
+				      (uint) pa->max_length,
 				      MYF(MY_WME))))
       DBUG_RETURN(1);
     if (new_pos != pa->str)
@@ -286,7 +293,6 @@ int insert_pointer_name(reg1 POINTER_ARRAY *pa,my_string name)
 					      char*);
       pa->str=new_pos;
     }
-    pa->max_length+=PS_MALLOC;
   }
   if (pa->typelib.count >= pa->max_count-1)
   {
@@ -370,8 +376,8 @@ static REP_SET *make_new_set(REP_SETS *sets);
 static void make_sets_invisible(REP_SETS *sets);
 static void free_last_set(REP_SETS *sets);
 static void free_sets(REP_SETS *sets);
-static void set_bit(REP_SET *set, uint bit);
-static void clear_bit(REP_SET *set, uint bit);
+static void internal_set_bit(REP_SET *set, uint bit);
+static void internal_clear_bit(REP_SET *set, uint bit);
 static void or_bits(REP_SET *to,REP_SET *from);
 static void copy_bits(REP_SET *to,REP_SET *from);
 static int cmp_bits(REP_SET *set1,REP_SET *set2);
@@ -448,7 +454,7 @@ REPLACE *init_replace(my_string *from, my_string *to,uint count,
   {
     if (from[i][0] == '\\' && from[i][1] == '^')
     {
-      set_bit(start_states,states+1);
+      internal_set_bit(start_states,states+1);
       if (!from[i][2])
       {
 	start_states->table_offset=i;
@@ -457,8 +463,8 @@ REPLACE *init_replace(my_string *from, my_string *to,uint count,
     }
     else if (from[i][0] == '\\' && from[i][1] == '$')
     {
-      set_bit(start_states,states);
-      set_bit(word_states,states);
+      internal_set_bit(start_states,states);
+      internal_set_bit(word_states,states);
       if (!from[i][2] && start_states->table_offset == (uint) ~0)
       {
 	start_states->table_offset=i;
@@ -467,11 +473,11 @@ REPLACE *init_replace(my_string *from, my_string *to,uint count,
     }
     else
     {
-      set_bit(word_states,states);
+      internal_set_bit(word_states,states);
       if (from[i][0] == '\\' && (from[i][1] == 'b' && from[i][2]))
-	set_bit(start_states,states+1);
+	internal_set_bit(start_states,states+1);
       else
-	set_bit(start_states,states);
+	internal_set_bit(start_states,states);
     }
     for (pos=from[i], len=0; *pos ; pos++)
     {
@@ -577,9 +583,9 @@ REPLACE *init_replace(my_string *from, my_string *to,uint count,
 		follow[i].len > found_end)
 	      found_end=follow[i].len;
 	    if (chr && follow[i].chr)
-	      set_bit(new_set,i+1);		/* To next set */
+	      internal_set_bit(new_set,i+1);		/* To next set */
 	    else
-	      set_bit(new_set,i);
+	      internal_set_bit(new_set,i);
 	  }
 	}
 	if (found_end)
@@ -596,7 +602,7 @@ REPLACE *init_replace(my_string *from, my_string *to,uint count,
 	    if (follow[bit_nr-1].len < found_end ||
 		(new_set->found_len &&
 		 (chr == 0 || !follow[bit_nr].chr)))
-	      clear_bit(new_set,i);
+	      internal_clear_bit(new_set,i);
 	    else
 	    {
 	      if (chr == 0 || !follow[bit_nr].chr)
@@ -745,13 +751,13 @@ static void free_sets(REP_SETS *sets)
   return;
 }
 
-static void set_bit(REP_SET *set, uint bit)
+static void internal_set_bit(REP_SET *set, uint bit)
 {
   set->bits[bit / WORD_BIT] |= 1 << (bit % WORD_BIT);
   return;
 }
 
-static void clear_bit(REP_SET *set, uint bit)
+static void internal_clear_bit(REP_SET *set, uint bit)
 {
   set->bits[bit / WORD_BIT] &= ~ (1 << (bit % WORD_BIT));
   return;
@@ -949,9 +955,11 @@ static void free_buffer()
 }
 
 
-/* Fill the buffer retaining the last n bytes at the beginning of the
-   newly filled buffer (for backward context).  Returns the number of new
-   bytes read from disk. */
+/*
+  Fill the buffer retaining the last n bytes at the beginning of the
+  newly filled buffer (for backward context).  Returns the number of new
+  bytes read from disk.
+*/
 
 static int fill_buffer_retaining(fd,n)
 File fd;
