@@ -201,14 +201,12 @@ the read requests for the whole area. */
 
 buf_pool_t*	buf_pool = NULL; /* The buffer buf_pool of the database */
 
-#ifdef UNIV_DEBUG
-static ulint	buf_dbg_counter	= 0; /* This is used to insert validation
+ulint		buf_dbg_counter	= 0; /* This is used to insert validation
 					operations in excution in the
 					debug version */
 ibool		buf_debug_prints = FALSE; /* If this is set TRUE,
 					the program prints info whenever
 					read-ahead or flush occurs */
-#endif /* UNIV_DEBUG */
 
 /************************************************************************
 Calculates a page checksum which is stored to the page when it is written
@@ -1457,12 +1455,10 @@ buf_page_create(
 
 	/* If we get here, the page was not in buf_pool: init it there */
 
-#ifdef UNIV_DEBUG
 	if (buf_debug_prints) {
 		fprintf(stderr, "Creating space %lu page %lu to buffer\n",
 			space, offset);
 	}
-#endif /* UNIV_DEBUG */
 
 	block = free_block;
 	
@@ -1565,9 +1561,9 @@ buf_page_io_complete(
 		"InnoDB: by dumping, dropping, and reimporting\n"
 		"InnoDB: the corrupt table. You can use CHECK\n"
 		"InnoDB: TABLE to scan your table for corruption.\n"
-		"InnoDB: Look also at section 6.1 of\n"
-		"InnoDB: http://www.innodb.com/ibman.php about\n"
-		"InnoDB: forcing recovery.\n", stderr);
+		"InnoDB: See also "
+		"http://dev.mysql.com/doc/mysql/en/Forcing_recovery.html\n"
+		"InnoDB: about forcing recovery.\n", stderr);
 			  
 			if (srv_force_recovery < SRV_FORCE_IGNORE_CORRUPT) {
 				fputs(
@@ -1613,11 +1609,9 @@ buf_page_io_complete(
 		rw_lock_x_unlock_gen(&(block->lock), BUF_IO_READ);
 		rw_lock_x_unlock_gen(&(block->read_lock), BUF_IO_READ);
 
-#ifdef UNIV_DEBUG
 		if (buf_debug_prints) {
 			fputs("Has read ", stderr);
 		}
-#endif /* UNIV_DEBUG */
 	} else {
 		ut_ad(io_type == BUF_IO_WRITE);
 
@@ -1630,21 +1624,17 @@ buf_page_io_complete(
 
 		buf_pool->n_pages_written++;
 
-#ifdef UNIV_DEBUG
 		if (buf_debug_prints) {
 			fputs("Has written ", stderr);
 		}
-#endif /* UNIV_DEBUG */
 	}
 	
 	mutex_exit(&(buf_pool->mutex));
 
-#ifdef UNIV_DEBUG
 	if (buf_debug_prints) {
 		fprintf(stderr, "page space %lu page no %lu\n",
 			block->space, block->offset);
 	}
-#endif /* UNIV_DEBUG */
 }
 
 /*************************************************************************
@@ -1673,7 +1663,6 @@ buf_pool_invalidate(void)
 	mutex_exit(&(buf_pool->mutex));
 }
 
-#ifdef UNIV_DEBUG
 /*************************************************************************
 Validates the buffer buf_pool data structure. */
 
@@ -1872,7 +1861,6 @@ buf_print(void)
 
 	ut_a(buf_validate());
 }	
-#endif /* UNIV_DEBUG */
 
 /*************************************************************************
 Returns the number of pending buf pool ios. */
