@@ -31,7 +31,6 @@ flushed along with the original page. */
 #define BUF_FLUSH_AREA		ut_min(BUF_READ_AHEAD_AREA,\
 					       buf_pool->curr_size / 16)
 
-#ifdef UNIV_DEBUG
 /**********************************************************************
 Validates the flush list. */
 static
@@ -39,7 +38,6 @@ ibool
 buf_flush_validate_low(void);
 /*========================*/
 		/* out: TRUE if ok */
-#endif /* UNIV_DEBUG */
 
 /************************************************************************
 Inserts a modified block into the flush list. */
@@ -215,7 +213,9 @@ buf_flush_buffered_writes(void)
 /*===========================*/
 {
 	buf_block_t*	block;
+	byte*		write_buf;
 	ulint		len;
+	ulint		len2;
 	ulint		i;
 
 	if (trx_doublewrite == NULL) {
@@ -242,6 +242,16 @@ buf_flush_buffered_writes(void)
 		block = trx_doublewrite->buf_block_arr[i];
 	        ut_a(block->state == BUF_BLOCK_FILE_PAGE);
 
+		if (mach_read_from_4(block->frame + FIL_PAGE_LSN + 4)
+                            != mach_read_from_4(block->frame + UNIV_PAGE_SIZE
+                                        - FIL_PAGE_END_LSN_OLD_CHKSUM + 4)) {
+                            ut_print_timestamp(stderr);
+                            fprintf(stderr,
+"  InnoDB: ERROR: The page to be written seems corrupt!\n"
+"InnoDB: The lsn fields do not match! Noticed in the buffer pool\n"
+"InnoDB: before posting to the doublewrite buffer.\n");
+                }
+
 		if (block->check_index_page_at_flush
 				&& !page_simple_validate(block->frame)) {
 
@@ -270,6 +280,19 @@ buf_flush_buffered_writes(void)
 		trx_doublewrite->block1, 0, len,
 		 	(void*)trx_doublewrite->write_buf, NULL);
 	
+	write_buf = trx_doublewrite->write_buf;
+
+        for (len2 = 0; len2 + UNIV_PAGE_SIZE <= len; len2 += UNIV_PAGE_SIZE) {
+        	if (mach_read_from_4(write_buf + len2 + FIL_PAGE_LSN + 4)
+                    != mach_read_from_4(write_buf + len2 + UNIV_PAGE_SIZE
+                                        - FIL_PAGE_END_LSN_OLD_CHKSUM + 4)) {
+			ut_print_timestamp(stderr);
+			fprintf(stderr,
+"  InnoDB: ERROR: The page to be written seems corrupt!\n"
+"InnoDB: The lsn fields do not match! Noticed in the doublewrite block1.\n");
+		}
+	}
+
 	if (trx_doublewrite->first_free > TRX_SYS_DOUBLEWRITE_BLOCK_SIZE) {
 		len = (trx_doublewrite->first_free
 			- TRX_SYS_DOUBLEWRITE_BLOCK_SIZE) * UNIV_PAGE_SIZE;
@@ -280,6 +303,22 @@ buf_flush_buffered_writes(void)
 		 	(void*)(trx_doublewrite->write_buf
 		 	+ TRX_SYS_DOUBLEWRITE_BLOCK_SIZE * UNIV_PAGE_SIZE),
 			NULL);
+
+		write_buf = trx_doublewrite->write_buf
+			   + TRX_SYS_DOUBLEWRITE_BLOCK_SIZE * UNIV_PAGE_SIZE;
+		for (len2 = 0; len2 + UNIV_PAGE_SIZE <= len;
+						len2 += UNIV_PAGE_SIZE) {
+        		if (mach_read_from_4(write_buf + len2
+							+ FIL_PAGE_LSN + 4)
+                    	    != mach_read_from_4(write_buf + len2
+					+ UNIV_PAGE_SIZE
+                                        - FIL_PAGE_END_LSN_OLD_CHKSUM + 4)) {
+				ut_print_timestamp(stderr);
+				fprintf(stderr,
+"  InnoDB: ERROR: The page to be written seems corrupt!\n"
+"InnoDB: The lsn fields do not match! Noticed in the doublewrite block2.\n");
+			}
+		}
 	}
 
 	/* Now flush the doublewrite buffer data to disk */
@@ -293,6 +332,19 @@ buf_flush_buffered_writes(void)
 	for (i = 0; i < trx_doublewrite->first_free; i++) {
 		block = trx_doublewrite->buf_block_arr[i];
 
+		if (mach_read_from_4(block->frame + FIL_PAGE_LSN + 4)
+                            != mach_read_from_4(block->frame + UNIV_PAGE_SIZE
+                                        - FIL_PAGE_END_LSN_OLD_CHKSUM + 4)) {
+                            ut_print_timestamp(stderr);
+                            fprintf(stderr,
+"  InnoDB: ERROR: The page to be written seems corrupt!\n"
+"InnoDB: The lsn fields do not match! Noticed in the buffer pool\n"
+"InnoDB: after posting and flushing the doublewrite buffer.\n"
+"InnoDB: Page buf fix count %lu, io fix %lu, state %lu\n",
+			(ulong)block->buf_fix_count, (ulong)block->io_fix,
+			(ulong)block->state);
+                }
+
 		fil_io(OS_FILE_WRITE | OS_AIO_SIMULATED_WAKE_LATER,
 			FALSE, block->space, block->offset, 0, UNIV_PAGE_SIZE,
 		 			(void*)block->frame, (void*)block);
@@ -490,13 +542,11 @@ buf_flush_try_page(
 			rw_lock_s_lock_gen(&(block->lock), BUF_IO_WRITE);
 		}
 
-#ifdef UNIV_DEBUG
 		if (buf_debug_prints) {
 			fprintf(stderr,
 				"Flushing page space %lu, page no %lu \n",
 				block->space, block->offset);
 		}
-#endif /* UNIV_DEBUG */
 
 		buf_flush_write_block_low(block);
 		
@@ -552,13 +602,11 @@ buf_flush_try_page(
 
 		rw_lock_s_lock_gen(&(block->lock), BUF_IO_WRITE);
 
-#ifdef UNIV_DEBUG
 		if (buf_debug_prints) {
 			fprintf(stderr,
 			"Flushing single page space %lu, page no %lu \n",
 						block->space, block->offset);
 		}
-#endif /* UNIV_DEBUG */
 
 		buf_flush_write_block_low(block);
 		
@@ -785,7 +833,6 @@ buf_flush_batch(
 
 	buf_flush_buffered_writes();
 
-#ifdef UNIV_DEBUG
 	if (buf_debug_prints && page_count > 0) {
 		ut_a(flush_type == BUF_FLUSH_LRU
 			|| flush_type == BUF_FLUSH_LIST);
@@ -794,7 +841,6 @@ buf_flush_batch(
 			: "Flushed %lu pages in flush list flush\n",
 			page_count);
 	}
-#endif /* UNIV_DEBUG */
 	
 	return(page_count);
 }
@@ -886,7 +932,6 @@ buf_flush_free_margin(void)
 	}
 }
 
-#ifdef UNIV_DEBUG
 /**********************************************************************
 Validates the flush list. */
 static
@@ -936,4 +981,3 @@ buf_flush_validate(void)
 
 	return(ret);
 }
-#endif /* UNIV_DEBUG */
