@@ -15,7 +15,6 @@ Created 1/16/1996 Heikki Tuuri
 dtype_t		dtype_binary_val = {DATA_BINARY, 0, 0, 0};
 dtype_t* 	dtype_binary 	= &dtype_binary_val;
 
-#ifdef UNIV_DEBUG
 /*************************************************************************
 Validates a data type structure. */
 
@@ -34,7 +33,6 @@ dtype_validate(
 
 	return(TRUE);
 }
-#endif /* UNIV_DEBUG */
 
 /*************************************************************************
 Prints a data type structure. */
