@@ -26,6 +26,8 @@ Created 1/8/1996 Heikki Tuuri
 #include "ut0byte.h"
 #include "trx0types.h"
 
+extern ibool		dict_char_0xA0_is_space;
+
 /************************************************************************
 Get the database name length in a table name. */
 
@@ -301,14 +303,6 @@ dict_table_get_index_noninline(
 	dict_table_t*	table,	/* in: table */
 	char*		name);	/* in: index name */
 /**************************************************************************
-Prints a table data. */
-
-void
-dict_table_print_low(
-/*=================*/
-	dict_table_t*	table);	/* in: table */
-#ifdef UNIV_DEBUG
-/**************************************************************************
 Prints a table definition. */
 
 void
@@ -316,13 +310,19 @@ dict_table_print(
 /*=============*/
 	dict_table_t*	table);	/* in: table */
 /**************************************************************************
+Prints a table data. */
+
+void
+dict_table_print_low(
+/*=================*/
+	dict_table_t*	table);	/* in: table */
+/**************************************************************************
 Prints a table data when we know the table name. */
 
 void
 dict_table_print_by_name(
 /*=====================*/
 	char*	name);
-#endif /* UNIV_DEBUG */
 /**************************************************************************
 Outputs info on foreign keys of a table. */
 
@@ -568,8 +568,10 @@ dict_index_contains_col_or_prefix(
 	dict_index_t*	index,	/* in: index */
 	ulint		n);	/* in: column number */
 /************************************************************************
-Looks for a matching field in an index. The column and the prefix len has
-to be the same. */
+Looks for a matching field in an index. The column has to be the same. The
+column in index must be complete, or must contain a prefix longer than the
+column in index2. That is, we must be able to construct the prefix in index2
+from the prefix in index. */
 
 ulint
 dict_index_get_nth_field_pos(
