@@ -31,8 +31,10 @@ ulint	log_fsp_current_free_limit		= 0;
 log_t*	log_sys	= NULL;
 
 ibool	log_do_write = TRUE;
+
 ibool	log_debug_writes = FALSE;
 
+
 /* These control how often we print warnings if the last checkpoint is too
 old */
 ibool	log_has_printed_chkp_warning = FALSE;
@@ -85,9 +87,8 @@ the previous */
 Completes a checkpoint write i/o to a log file. */
 static
 void
-log_io_complete_checkpoint(
-/*=======================*/
-	log_group_t*	group);	/* in: log group */
+log_io_complete_checkpoint(void);
+/*============================*/
 /**********************************************************
 Completes an archiving i/o. */
 static
@@ -140,7 +141,9 @@ log_buf_pool_get_oldest_modification(void)
 {
 	dulint	lsn;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	lsn = buf_pool_get_oldest_modification();
 
@@ -239,7 +242,9 @@ log_write_low(
 	ulint	data_len;
 	byte*	log_block;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 part_loop:
 	/* Calculate a part length */
 
@@ -306,7 +311,9 @@ log_close(void)
 	log_t*	log	= log_sys;
 	ulint	checkpoint_age;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	lsn = log->lsn;
 	
@@ -407,7 +414,7 @@ log_pad_current_log_block(void)
 	log_close();
 	log_release();
 
-	ut_anp((ut_dulint_get_low(lsn) % OS_FILE_LOG_BLOCK_SIZE)
+	ut_a((ut_dulint_get_low(lsn) % OS_FILE_LOG_BLOCK_SIZE)
 						== LOG_BLOCK_HDR_SIZE);
 }
 
@@ -421,7 +428,9 @@ log_group_get_capacity(
 				/* out: capacity in bytes */
 	log_group_t*	group)	/* in: log group */
 {
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	return((group->file_size - LOG_FILE_HDR_SIZE) * group->n_files); 
 }
@@ -437,7 +446,9 @@ log_group_calc_size_offset(
 	ulint		offset,	/* in: real offset within the log group */
 	log_group_t*	group)	/* in: log group */
 {
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	return(offset - LOG_FILE_HDR_SIZE * (1 + offset / group->file_size));
 }
@@ -453,7 +464,9 @@ log_group_calc_real_offset(
 	ulint		offset,	/* in: size offset within the log group */
 	log_group_t*	group)	/* in: log group */
 {
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	return(offset + LOG_FILE_HDR_SIZE
 		* (1 + offset / (group->file_size - LOG_FILE_HDR_SIZE)));
@@ -475,7 +488,9 @@ log_group_calc_lsn_offset(
 	ib_longlong	group_size;
 	ib_longlong	offset;
 	
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	/* If total log file size is > 2 GB we can easily get overflows
 	with 32-bit integers. Use 64-bit integers instead. */
@@ -502,7 +517,8 @@ log_group_calc_lsn_offset(
 
 	ut_a(offset < (((ib_longlong) 1) << 32)); /* offset must be < 4 GB */
 
-	/* printf("Offset is %lu gr_lsn_offset is %lu difference is %lu\n",
+	/* fprintf(stderr,
+		"Offset is %lu gr_lsn_offset is %lu difference is %lu\n",
 	       (ulint)offset,(ulint)gr_lsn_size_offset, (ulint)difference);
 	*/
 
@@ -589,7 +605,9 @@ log_calc_max_ages(void)
 	ulint		archive_margin;
 	ulint		smallest_archive_margin;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(!mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	mutex_enter(&(log_sys->mutex));
 
@@ -661,7 +679,17 @@ failure:
 
 	if (!success) {
 		fprintf(stderr,
-"InnoDB: Error: log file group too small for innodb_thread_concurrency\n");
+"InnoDB: Error: ib_logfiles are too small for innodb_thread_concurrency %lu.\n"
+"InnoDB: The combined size of ib_logfiles should be bigger than\n"
+"InnoDB: 200 kB * innodb_thread_concurrency.\n"
+"InnoDB: To get mysqld to start up, set innodb_thread_concurrency in my.cnf\n"
+"InnoDB: to a lower value, for example, to 8. After an ERROR-FREE shutdown\n"
+"InnoDB: of mysqld you can adjust the size of ib_logfiles, as explained in\n"
+"InnoDB: http://dev.mysql.com/doc/mysql/en/Adding_and_removing.html\n"
+"InnoDB: Cannot continue operation. Calling exit(1).\n",
+			(ulong)srv_thread_concurrency);
+
+		exit(1);
 	}
 
 	return(success);
@@ -764,11 +792,7 @@ log_init(void)
 	log_sys->archiving_on = os_event_create(NULL);
 
 	/*----------------------------*/
-	
-	log_sys->online_backup_state = FALSE;
 
-	/*----------------------------*/
-	
 	log_block_init(log_sys->buf, log_sys->lsn);
 	log_block_set_first_rec_group(log_sys->buf, LOG_BLOCK_HDR_SIZE);
 
@@ -865,7 +889,9 @@ log_flush_do_unlocks(
 	ulint	code)	/* in: any ORed combination of LOG_UNLOCK_FLUSH_LOCK
 			and LOG_UNLOCK_NONE_FLUSHED_LOCK */
 {
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	/* NOTE that we must own the log mutex when doing the setting of the
 	events: this is because transactions will wait for these events to
@@ -897,12 +923,14 @@ log_group_check_flush_completion(
 				/* out: LOG_UNLOCK_NONE_FLUSHED_LOCK or 0 */
 	log_group_t*	group)	/* in: log group */
 {
-	ut_ad(mutex_own(&(log_sys->mutex)));	
+#ifdef UNIV_SYNC_DEBUG
+	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	if (!log_sys->one_flushed && group->n_pending_writes == 0) {
-
 		if (log_debug_writes) {
-			printf("Log flushed first to group %lu\n", group->id);
+			fprintf(stderr,
+				"Log flushed first to group %lu\n", group->id);
 		}
 	
 		log_sys->written_to_some_lsn = log_sys->write_lsn;
@@ -913,7 +941,7 @@ log_group_check_flush_completion(
 
 	if (log_debug_writes && (group->n_pending_writes == 0)) {
 
-		printf("Log flushed to group %lu\n", group->id);
+		fprintf(stderr, "Log flushed to group %lu\n", group->id);
 	}
 
 	return(0);
@@ -930,7 +958,9 @@ log_sys_check_flush_completion(void)
 	ulint	move_start;
 	ulint	move_end;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	if (log_sys->n_pending_writes == 0) {
 	
@@ -978,7 +1008,7 @@ log_io_complete(
 		return;
 	}
 
-	if ((ulint)group & 0x1) {
+	if ((ulint)group & 0x1UL) {
 		/* It was a checkpoint write */
 		group = (log_group_t*)((ulint)group - 1);
 
@@ -988,12 +1018,18 @@ log_io_complete(
 		        fil_flush(group->space_id);
 		}
 
-		log_io_complete_checkpoint(group);
+		if (log_debug_writes) {
+			fprintf(stderr,
+				"Checkpoint info written to group %lu\n",
+				group->id);
+		}
+
+		log_io_complete_checkpoint();
 
 		return;
 	}
 
-	ut_a(0);	/* We currently use synchronous writing of the
+	ut_error;	/* We currently use synchronous writing of the
 			logs and cannot end up here! */
 
 	if (srv_unix_file_flush_method != SRV_UNIX_O_DSYNC
@@ -1025,7 +1061,6 @@ static
 void
 log_group_file_header_flush(
 /*========================*/
-	ulint		type,		/* in: LOG_FLUSH or LOG_RECOVER */
 	log_group_t*	group,		/* in: log group */
 	ulint		nth_file,	/* in: header to the nth file in the
 					log file space */
@@ -1034,10 +1069,9 @@ log_group_file_header_flush(
 {
 	byte*	buf;
 	ulint	dest_offset;
-	
-	UT_NOT_USED(type);
-
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	ut_a(nth_file < group->n_files);
 
@@ -1052,9 +1086,9 @@ log_group_file_header_flush(
 	dest_offset = nth_file * group->file_size;
 
 	if (log_debug_writes) {
-		printf(
-		"Writing log file header to group %lu file %lu\n", group->id,
-								nth_file);
+		fprintf(stderr,
+			"Writing log file header to group %lu file %lu\n",
+			group->id, nth_file);
 	}
 
 	if (log_do_write) {
@@ -1087,7 +1121,6 @@ Writes a buffer to a log file group. */
 void
 log_group_write_buf(
 /*================*/
-	ulint		type,		/* in: LOG_FLUSH or LOG_RECOVER */
 	log_group_t*	group,		/* in: log group */
 	byte*		buf,		/* in: buffer */
 	ulint		len,		/* in: buffer len; must be divisible
@@ -1105,9 +1138,11 @@ log_group_write_buf(
 	ulint	next_offset;
 	ulint	i;
 	
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
-	ut_anp(len % OS_FILE_LOG_BLOCK_SIZE == 0);
-	ut_anp(ut_dulint_get_low(start_lsn) % OS_FILE_LOG_BLOCK_SIZE == 0);
+#endif /* UNIV_SYNC_DEBUG */
+	ut_a(len % OS_FILE_LOG_BLOCK_SIZE == 0);
+	ut_a(ut_dulint_get_low(start_lsn) % OS_FILE_LOG_BLOCK_SIZE == 0);
 
 	if (new_data_offset == 0) {
 		write_header = TRUE;
@@ -1126,7 +1161,7 @@ loop:
 	   						&& write_header) {
 		/* We start to write a new log file instance in the group */
 
-		log_group_file_header_flush(type, group,
+		log_group_file_header_flush(group,
 				next_offset / group->file_size, start_lsn);
 	}
 
@@ -1139,14 +1174,13 @@ loop:
 	
 	if (log_debug_writes) {
 
-		printf(
+		fprintf(stderr,
 		"Writing log file segment to group %lu offset %lu len %lu\n"
-		"start lsn %lu %lu\n",
+		"start lsn %lu %lu\n"
+		"First block n:o %lu last block n:o %lu\n",
 			group->id, next_offset, write_len,
 			ut_dulint_get_high(start_lsn),
-			ut_dulint_get_low(start_lsn));
-		printf(
-		"First block n:o %lu last block n:o %lu\n",
+			ut_dulint_get_low(start_lsn),
 			log_block_get_hdr_no(buf),
 			log_block_get_hdr_no(
 				buf + write_len - OS_FILE_LOG_BLOCK_SIZE));
@@ -1226,7 +1260,7 @@ loop:
 	ut_ad(loop_count < 5);
 
 	if (loop_count > 2) {
-/*		printf("Log loop count %lu\n", loop_count); */
+/*		fprintf(stderr, "Log loop count %lu\n", loop_count); */
 	}
 	
 	mutex_enter(&(log_sys->mutex));
@@ -1288,7 +1322,8 @@ loop:
 	}
 
 	if (log_debug_writes) {
-		printf("Writing log from %lu %lu up to lsn %lu %lu\n",
+		fprintf(stderr,
+			"Writing log from %lu %lu up to lsn %lu %lu\n",
 			ut_dulint_get_high(log_sys->written_to_all_lsn),
 			ut_dulint_get_low(log_sys->written_to_all_lsn),
 					ut_dulint_get_high(log_sys->lsn),
@@ -1341,7 +1376,7 @@ loop:
 	/* Do the write to the log files */
 
 	while (group) {
-		log_group_write_buf(LOG_FLUSH, group,
+		log_group_write_buf(group,
 			log_sys->buf + area_start,
 			area_end - area_start,
 			ut_dulint_align_down(log_sys->written_to_all_lsn,
@@ -1502,7 +1537,9 @@ void
 log_complete_checkpoint(void)
 /*=========================*/
 {
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 	ut_ad(log_sys->n_pending_checkpoint_writes == 0);
 
 	log_sys->next_checkpoint_no
@@ -1517,9 +1554,8 @@ log_complete_checkpoint(void)
 Completes an asynchronous checkpoint info write i/o to a log file. */
 static
 void
-log_io_complete_checkpoint(
-/*=======================*/
-	log_group_t*	group)	/* in: log group */
+log_io_complete_checkpoint(void)
+/*============================*/
 {
 	mutex_enter(&(log_sys->mutex));
 
@@ -1527,10 +1563,6 @@ log_io_complete_checkpoint(
 	
 	log_sys->n_pending_checkpoint_writes--;
 
-	if (log_debug_writes) {
-		printf("Checkpoint info written to group %lu\n", group->id);
-	}
-
 	if (log_sys->n_pending_checkpoint_writes == 0) {
 		log_complete_checkpoint();
 	}	
@@ -1592,7 +1624,9 @@ log_group_checkpoint(
 	byte*	buf;
 	ulint	i;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 	ut_a(LOG_CHECKPOINT_SIZE <= OS_FILE_LOG_BLOCK_SIZE);
 	
 	buf = group->checkpoint_buf;
@@ -1708,11 +1742,11 @@ log_reset_first_header_and_checkpoint(
 	lsn = ut_dulint_add(start, LOG_BLOCK_HDR_SIZE);
 
 	/* Write the label of ibbackup --restore */
-	sprintf((char*) hdr_buf + LOG_FILE_WAS_CREATED_BY_HOT_BACKUP,
+	strcpy((char*) hdr_buf + LOG_FILE_WAS_CREATED_BY_HOT_BACKUP,
 				"ibbackup ");
 	ut_sprintf_timestamp(
-			(char*) hdr_buf + LOG_FILE_WAS_CREATED_BY_HOT_BACKUP
-						+ strlen("ibbackup "));
+			(char*) hdr_buf + (LOG_FILE_WAS_CREATED_BY_HOT_BACKUP
+						+ (sizeof "ibbackup ") - 1));
 	buf = hdr_buf + LOG_CHECKPOINT_1;
 	
 	mach_write_to_8(buf + LOG_CHECKPOINT_NO, ut_dulint_zero);
@@ -1746,7 +1780,9 @@ log_group_read_checkpoint_info(
 	log_group_t*	group,	/* in: log group */
 	ulint		field)	/* in: LOG_CHECKPOINT_1 or LOG_CHECKPOINT_2 */
 {
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	log_sys->n_log_ios++;
 	
@@ -1764,7 +1800,9 @@ log_groups_write_checkpoint_info(void)
 {
 	log_group_t*	group;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	group = UT_LIST_GET_FIRST(log_sys->log_groups);
 
@@ -1850,7 +1888,7 @@ log_checkpoint(
 	log_sys->next_checkpoint_lsn = oldest_lsn;
 
 	if (log_debug_writes) {
-		printf("Making checkpoint no %lu at lsn %lu %lu\n",
+		fprintf(stderr, "Making checkpoint no %lu at lsn %lu %lu\n",
 			ut_dulint_get_low(log_sys->next_checkpoint_no),
 			ut_dulint_get_high(oldest_lsn),
 			ut_dulint_get_low(oldest_lsn));
@@ -2029,7 +2067,9 @@ log_group_read_log_seg(
 	ulint	source_offset;
 	ibool	sync;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	sync = FALSE;
 
@@ -2075,11 +2115,11 @@ void
 log_archived_file_name_gen(
 /*=======================*/
 	char*	buf,	/* in: buffer where to write */
-	ulint	id,	/* in: group id */
+	ulint	id __attribute__((unused)),
+			/* in: group id;
+			currently we only archive the first group */
 	ulint	file_no)/* in: file number */
 {
-	UT_NOT_USED(id);	/* Currently we only archive the first group */
-	
 	sprintf(buf, "%sib_arch_log_%010lu", srv_arch_dir, file_no);
 }
 
@@ -2099,7 +2139,9 @@ log_group_archive_file_header_write(
 	byte*	buf;
 	ulint	dest_offset;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	ut_a(nth_file < group->n_files);
 
@@ -2136,7 +2178,9 @@ log_group_archive_completed_header_write(
 	byte*	buf;
 	ulint	dest_offset;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 	ut_a(nth_file < group->n_files);
 
 	buf = *(group->archive_file_header_bufs + nth_file);
@@ -2175,15 +2219,17 @@ log_group_archive(
 	ulint	n_files;
 	ulint	open_mode;
 	
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	start_lsn = log_sys->archived_lsn;
 
-	ut_anp(ut_dulint_get_low(start_lsn) % OS_FILE_LOG_BLOCK_SIZE == 0);
+	ut_a(ut_dulint_get_low(start_lsn) % OS_FILE_LOG_BLOCK_SIZE == 0);
 
 	end_lsn = log_sys->next_archived_lsn;
 
-	ut_anp(ut_dulint_get_low(end_lsn) % OS_FILE_LOG_BLOCK_SIZE == 0);
+	ut_a(ut_dulint_get_low(end_lsn) % OS_FILE_LOG_BLOCK_SIZE == 0);
 
 	buf = log_sys->archive_buf;
 
@@ -2216,18 +2262,17 @@ loop:
 		}
 
 		if (!ret) {
-		  fprintf(stderr,
-		   "InnoDB: Cannot create or open archive log file %s.\n",
-			  name);
-		  fprintf(stderr, "InnoDB: Cannot continue operation.\n"
-       		  "InnoDB: Check that the log archive directory exists,\n"
-			  "InnoDB: you have access rights to it, and\n"
-			  "InnoDB: there is space available.\n");
-		  exit(1);
+			fprintf(stderr,
+		"InnoDB: Cannot create or open archive log file %s.\n"
+		"InnoDB: Cannot continue operation.\n"
+		"InnoDB: Check that the log archive directory exists,\n"
+		"InnoDB: you have access rights to it, and\n"
+		"InnoDB: there is space available.\n", name);
+			exit(1);
 		}
 
 		if (log_debug_writes) {
-			printf("Created archive file %s\n", name);
+			fprintf(stderr, "Created archive file %s\n", name);
 		}
 
 		ret = os_file_close(file_handle);
@@ -2258,7 +2303,7 @@ loop:
 	}
 	
 	if (log_debug_writes) {
-		printf(
+		fprintf(stderr,
 		"Archiving starting at lsn %lu %lu, len %lu to group %lu\n",
 					ut_dulint_get_high(start_lsn),
 					ut_dulint_get_low(start_lsn),
@@ -2290,7 +2335,7 @@ loop:
 	group->next_archived_file_no = group->archived_file_no + n_files;
 	group->next_archived_offset = next_offset % group->file_size;
 
-	ut_anp(group->next_archived_offset % OS_FILE_LOG_BLOCK_SIZE == 0);
+	ut_a(group->next_archived_offset % OS_FILE_LOG_BLOCK_SIZE == 0);
 }
 
 /*********************************************************
@@ -2303,7 +2348,9 @@ log_archive_groups(void)
 {
 	log_group_t*	group;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	group = UT_LIST_GET_FIRST(log_sys->log_groups);
 
@@ -2326,7 +2373,9 @@ log_archive_write_complete_groups(void)
 	dulint		end_lsn;
 	ulint		i;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	group = UT_LIST_GET_FIRST(log_sys->log_groups);
 
@@ -2351,7 +2400,8 @@ log_archive_write_complete_groups(void)
 	}
 
 	if (log_debug_writes && trunc_files) {
-		printf("Complete file(s) archived to group %lu\n",
+		fprintf(stderr,
+			"Complete file(s) archived to group %lu\n",
 								group->id);
 	}
 
@@ -2377,7 +2427,7 @@ log_archive_write_complete_groups(void)
 					trunc_files * group->file_size);
 
 	if (log_debug_writes) {
-		printf("Archiving writes completed\n");
+		fputs("Archiving writes completed\n", stderr);
 	}
 }
 
@@ -2388,13 +2438,15 @@ void
 log_archive_check_completion_low(void)
 /*==================================*/
 {
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	if (log_sys->n_pending_archive_ios == 0
 			&& log_sys->archiving_phase == LOG_ARCHIVE_READ) {
 
 		if (log_debug_writes) {
-			printf("Archiving read completed\n");
+			fputs("Archiving read completed\n", stderr);
 		}
 
 	    	/* Archive buffer has now been read in: start archive writes */
@@ -2485,8 +2537,7 @@ loop:
 	start_lsn = log_sys->archived_lsn;
 	
 	if (calc_new_limit) {
-		ut_anp(log_sys->archive_buf_size % OS_FILE_LOG_BLOCK_SIZE
-								== 0);
+		ut_a(log_sys->archive_buf_size % OS_FILE_LOG_BLOCK_SIZE == 0);
 		limit_lsn = ut_dulint_add(start_lsn,
 						log_sys->archive_buf_size);
 
@@ -2541,7 +2592,8 @@ loop:
 	log_sys->next_archived_lsn = limit_lsn;
 
 	if (log_debug_writes) {
-		printf("Archiving from lsn %lu %lu to lsn %lu %lu\n",
+		fprintf(stderr,
+			"Archiving from lsn %lu %lu to lsn %lu %lu\n",
 			ut_dulint_get_high(log_sys->archived_lsn),
 			ut_dulint_get_low(log_sys->archived_lsn),
 			ut_dulint_get_high(limit_lsn),
@@ -2620,7 +2672,9 @@ log_archive_close_groups(
 	log_group_t*	group;
 	ulint		trunc_len;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	group = UT_LIST_GET_FIRST(log_sys->log_groups);
 
@@ -2644,7 +2698,7 @@ log_archive_close_groups(
 		}
 
 		if (log_debug_writes) {
-			printf(
+			fprintf(stderr,
 			"Incrementing arch file no to %lu in log group %lu\n",
 				group->archived_file_no + 2, group->id);
 		}
@@ -2886,66 +2940,6 @@ loop:
 	mutex_exit(&(log_sys->mutex));
 }
 
-/**********************************************************
-Switches the database to the online backup state. */
-
-ulint
-log_switch_backup_state_on(void)
-/*============================*/
-			/* out: DB_SUCCESS or DB_ERROR */
-{
-	dulint	backup_lsn;
-	
-	mutex_enter(&(log_sys->mutex));
-
-	if (log_sys->online_backup_state) {
-
-		/* The database is already in that state */
-
-		mutex_exit(&(log_sys->mutex));
-
-		return(DB_ERROR);
-	}
-
-	log_sys->online_backup_state = TRUE;
-
-	backup_lsn = log_sys->lsn;
-
-	log_sys->online_backup_lsn = backup_lsn;
-
-	mutex_exit(&(log_sys->mutex));
-
-	/* log_checkpoint_and_mark_file_spaces(); */
-
-	return(DB_SUCCESS);
-}
-
-/**********************************************************
-Switches the online backup state off. */
-
-ulint
-log_switch_backup_state_off(void)
-/*=============================*/
-			/* out: DB_SUCCESS or DB_ERROR */
-{
-	mutex_enter(&(log_sys->mutex));
-
-	if (!log_sys->online_backup_state) {
-
-		/* The database is already in that state */
-
-		mutex_exit(&(log_sys->mutex));
-
-		return(DB_ERROR);
-	}
-
-	log_sys->online_backup_state = FALSE;
-
-	mutex_exit(&(log_sys->mutex));
-
-	return(DB_SUCCESS);
-}
-
 /********************************************************************
 Makes a checkpoint at the latest lsn and writes it to first page of each
 data file in the database, so that we know that the file spaces contain
@@ -3114,7 +3108,9 @@ log_check_log_recs(
 	byte*	buf1;
 	byte*	scan_buf;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(log_sys->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	if (len == 0) {
 
@@ -3174,20 +3170,15 @@ Prints info of the log. */
 void
 log_print(
 /*======*/
-	char*	buf,	/* in/out: buffer where to print */
-	char*	buf_end)/* in: buffer end */
+	FILE*	file)	/* in: file where to print */
 {
 	double	time_elapsed;
 	time_t	current_time;
 
-	if (buf_end - buf < 300) {
-
-		return;
-	}
-
 	mutex_enter(&(log_sys->mutex));
 
-	buf += sprintf(buf, "Log sequence number %lu %lu\n"
+	fprintf(file,
+		"Log sequence number %lu %lu\n"
 	       "Log flushed up to   %lu %lu\n"
 	       "Last checkpoint at  %lu %lu\n",
 			ut_dulint_get_high(log_sys->lsn),
@@ -3201,7 +3192,7 @@ log_print(
 			
 	time_elapsed = 0.001 + difftime(current_time,
 					log_sys->last_printout_time);
-	buf += sprintf(buf,
+	fprintf(file,
 	"%lu pending log writes, %lu pending chkp writes\n"
 	"%lu log i/o's done, %.2f log i/o's/second\n",
 	log_sys->n_pending_writes,
