@@ -214,6 +214,7 @@ que_thr_end_wait(
 	if (next_thr && *next_thr == NULL) {
 		*next_thr = thr;
 	} else {
+		ut_a(0);
 		srv_que_task_enqueue_low(thr);
 	}
 }	
@@ -394,6 +395,7 @@ que_fork_error_handle(
 	
 	que_thr_move_to_run_state(thr);
 
+	ut_a(0);
 	srv_que_task_enqueue_low(thr);
 }
 
@@ -804,6 +806,7 @@ que_thr_dec_refer_count(
 			if (next_thr && *next_thr == NULL) {
 				*next_thr = thr;
 			} else {
+				ut_a(0);
 				srv_que_task_enqueue_low(thr);
 			}
 
@@ -1034,10 +1037,9 @@ que_thr_stop_for_mysql_no_error(
 	trx->n_active_thrs--;
 }
 
-#ifdef UNIV_DEBUG
 /**************************************************************************
 Prints info of an SQL query graph node. */
-static
+
 void
 que_node_print_info(
 /*================*/
@@ -1094,7 +1096,6 @@ que_node_print_info(
 
 	fprintf(stderr, "Node type %lu: %s, address %p\n", type, str, node);
 }
-#endif /* UNIV_DEBUG */
 
 /**************************************************************************
 Performs an execution step on a query thread. */
@@ -1259,6 +1260,7 @@ loop:
 	loop_count++;
 
 	if (next_thr != thr) {
+		ut_a(next_thr == NULL);
 		que_thr_dec_refer_count(thr, &next_thr);
 
 		if (next_thr == NULL) {
