@@ -95,14 +95,11 @@ row_undo_mod_clust_low(
 	ulint		mode)	/* in: BTR_MODIFY_LEAF or BTR_MODIFY_TREE */
 {
 	big_rec_t*	dummy_big_rec;
-	dict_index_t*	index;
 	btr_pcur_t*	pcur;
 	btr_cur_t*	btr_cur;
 	ulint		err;
 	ibool		success;
 
-	index = dict_table_get_first_index(node->table);
-	
 	pcur = &(node->pcur);
 	btr_cur = btr_pcur_get_btr_cur(pcur);
 
@@ -317,13 +314,6 @@ row_undo_mod_del_mark_or_remove_sec_low(
 	if (!found) {
 		/* Not found */
 
-		/* FIXME: remove printfs in the final version */
-
-		/* printf(
-		"--UNDO MOD: Record not found from page %lu index %s\n",
-			buf_frame_get_page_no(btr_cur_get_rec(btr_cur)),
-			index->name); */
-
 		btr_pcur_close(&pcur);
 		mtr_commit(&mtr);
 
@@ -377,8 +367,14 @@ row_undo_mod_del_mark_or_remove_sec_low(
 }
 
 /***************************************************************
-Delete marks or removes a secondary index entry if found. */
-UNIV_INLINE
+Delete marks or removes a secondary index entry if found.
+NOTE that if we updated the fields of a delete-marked secondary index record
+so that alphabetically they stayed the same, e.g., 'abc' -> 'aBc', we cannot
+return to the original values because we do not know them. But this should
+not cause problems because in row0sel.c, in queries we always retrieve the
+clustered index record or an earlier version of it, if the secondary index
+record through which we do the search is delete-marked. */
+static
 ulint
 row_undo_mod_del_mark_or_remove_sec(
 /*================================*/
@@ -403,57 +399,88 @@ row_undo_mod_del_mark_or_remove_sec(
 }
 
 /***************************************************************
-Delete unmarks a secondary index entry which must be found. */
+Delete unmarks a secondary index entry which must be found. It might not be
+delete-marked at the moment, but it does not harm to unmark it anyway. We also
+need to update the fields of the secondary index record if we updated its
+fields but alphabetically they stayed the same, e.g., 'abc' -> 'aBc'. */
 static
-void
-row_undo_mod_del_unmark_sec(
-/*========================*/
-	undo_node_t*	node,	/* in: row undo node */
+ulint
+row_undo_mod_del_unmark_sec_and_undo_update(
+/*========================================*/
+				/* out: DB_FAIL or DB_SUCCESS or
+				DB_OUT_OF_FILE_SPACE */
+	ulint		mode,	/* in: search mode: BTR_MODIFY_LEAF or
+				BTR_MODIFY_TREE */
 	que_thr_t*	thr,	/* in: query thread */
 	dict_index_t*	index,	/* in: index */
 	dtuple_t*	entry)	/* in: index entry */
 {
+	mem_heap_t*	heap;
 	btr_pcur_t	pcur;
-	btr_cur_t*	btr_cur;
-	ulint		err;
+	upd_t*		update;
+	ulint		err		= DB_SUCCESS;
 	ibool		found;
+	big_rec_t*	dummy_big_rec;
 	mtr_t		mtr;
-	char           	err_buf[1000];
 
-	UT_NOT_USED(node);
-	
 	log_free_check();
 	mtr_start(&mtr);
 	
-	found = row_search_index_entry(index, entry, BTR_MODIFY_LEAF, &pcur,
-									&mtr);
+	found = row_search_index_entry(index, entry, mode, &pcur, &mtr);
+
 	if (!found) {
-	  	fprintf(stderr,
-			"InnoDB: error in sec index entry del undo in\n"
-		  	"InnoDB: index %s table %s\n", index->name,
-		  	index->table->name);
-	  	dtuple_sprintf(err_buf, 900, entry);
-	  	fprintf(stderr, "InnoDB: tuple %s\n", err_buf);
-
-	  	rec_sprintf(err_buf, 900, btr_pcur_get_rec(&pcur));
-	  	fprintf(stderr, "InnoDB: record %s\n", err_buf);
-
-		trx_print(err_buf, thr_get_trx(thr));
-	  	fprintf(stderr,
-			"%s\nInnoDB: Make a detailed bug report and send it\n",
-			err_buf);
-	  	fprintf(stderr, "InnoDB: to mysql@lists.mysql.com\n");
-
+		fputs("InnoDB: error in sec index entry del undo in\n"
+			"InnoDB: ", stderr);
+		dict_index_name_print(stderr, index);
+		fputs("\n"
+			"InnoDB: tuple ", stderr);
+		dtuple_print(stderr, entry);
+		fputs("\n"
+			"InnoDB: record ", stderr);
+		rec_print(stderr, btr_pcur_get_rec(&pcur));
+		putc('\n', stderr);
+		trx_print(stderr, thr_get_trx(thr));
+		fputs("\n"
+"InnoDB: Submit a detailed bug report to http://bugs.mysql.com\n", stderr);
 	} else {
-         	btr_cur = btr_pcur_get_btr_cur(&pcur);
+		btr_cur_t*	btr_cur = btr_pcur_get_btr_cur(&pcur);
 
 	        err = btr_cur_del_mark_set_sec_rec(BTR_NO_LOCKING_FLAG,
 						btr_cur, FALSE, thr, &mtr);
-	        ut_ad(err == DB_SUCCESS);
+	        ut_a(err == DB_SUCCESS);
+		heap = mem_heap_create(100);
+
+		update = row_upd_build_sec_rec_difference_binary(index, entry,
+					btr_cur_get_rec(btr_cur), heap);
+	        if (upd_get_n_fields(update) == 0) {
+
+			/* Do nothing */
+		
+		} else if (mode == BTR_MODIFY_LEAF) {
+                	/* Try an optimistic updating of the record, keeping
+			changes within the page */
+
+                	err = btr_cur_optimistic_update(BTR_KEEP_SYS_FLAG
+							| BTR_NO_LOCKING_FLAG,
+ 						btr_cur, update, 0, thr, &mtr);
+                	if (err == DB_OVERFLOW || err == DB_UNDERFLOW) {
+                        	err = DB_FAIL;
+                	}
+       		} else  {
+                	ut_a(mode == BTR_MODIFY_TREE);
+                	err = btr_cur_pessimistic_update(BTR_KEEP_SYS_FLAG
+							| BTR_NO_LOCKING_FLAG,
+						btr_cur, &dummy_big_rec,
+						update, 0, thr, &mtr);
+        	}			
+
+		mem_heap_free(heap);
 	}
 
 	btr_pcur_close(&pcur);
 	mtr_commit(&mtr);
+
+	return(err);
 }
 
 /***************************************************************
@@ -501,13 +528,14 @@ static
 ulint
 row_undo_mod_del_mark_sec(
 /*======================*/
-				/* out: DB_SUCCESS */
+				/* out: DB_SUCCESS or DB_OUT_OF_FILE_SPACE */
 	undo_node_t*	node,	/* in: row undo node */
 	que_thr_t*	thr)	/* in: query thread */
 {
 	mem_heap_t*	heap;
 	dtuple_t*	entry;
 	dict_index_t*	index;
+	ulint		err;
 
 	heap = mem_heap_create(1024);
 
@@ -516,7 +544,21 @@ row_undo_mod_del_mark_sec(
 
 		entry = row_build_index_entry(node->row, index, heap);
 		
-		row_undo_mod_del_unmark_sec(node, thr, index, entry);
+		err = row_undo_mod_del_unmark_sec_and_undo_update(
+						BTR_MODIFY_LEAF,
+						thr, index, entry);
+		if (err == DB_FAIL) {
+			err = row_undo_mod_del_unmark_sec_and_undo_update(
+						BTR_MODIFY_TREE,
+						thr, index, entry);
+		}
+
+		if (err != DB_SUCCESS) {
+
+			mem_heap_free(heap);
+
+			return(err);
+		}
 
 		node->index = dict_table_get_next_index(node->index);
 	}
@@ -532,7 +574,7 @@ static
 ulint
 row_undo_mod_upd_exist_sec(
 /*=======================*/
-				/* out: DB_SUCCESS or error code */
+				/* out: DB_SUCCESS or DB_OUT_OF_FILE_SPACE */
 	undo_node_t*	node,	/* in: row undo node */
 	que_thr_t*	thr)	/* in: query thread */
 {
@@ -558,22 +600,48 @@ row_undo_mod_upd_exist_sec(
 			/* Build the newest version of the index entry */
 			entry = row_build_index_entry(node->row, index, heap);
 
+			/* NOTE that if we updated the fields of a
+			delete-marked secondary index record so that
+			alphabetically they stayed the same, e.g.,
+			'abc' -> 'aBc', we cannot return to the original
+			values because we do not know them. But this should
+			not cause problems because in row0sel.c, in queries
+			we always retrieve the clustered index record or an
+			earlier version of it, if the secondary index record
+			through which we do the search is delete-marked. */
+
 			err = row_undo_mod_del_mark_or_remove_sec(node, thr,
-							index, entry);
+								index, entry);
 			if (err != DB_SUCCESS) {
 				mem_heap_free(heap);
 
 				return(err);
 			}
-							
+
 			/* We may have to update the delete mark in the
 			secondary index record of the previous version of
-			the row */
+			the row. We also need to update the fields of
+			the secondary index record if we updated its fields
+			but alphabetically they stayed the same, e.g.,
+			'abc' -> 'aBc'. */
 
 			row_upd_index_replace_new_col_vals(entry, index,
 							node->update, NULL);
+			err = row_undo_mod_del_unmark_sec_and_undo_update(
+						BTR_MODIFY_LEAF,
+						thr, index, entry);
+			if (err == DB_FAIL) {
+				err =
+				   row_undo_mod_del_unmark_sec_and_undo_update(
+						BTR_MODIFY_TREE,
+						thr, index, entry);
+			}
 
-			row_undo_mod_del_unmark_sec(node, thr, index, entry);
+			if (err != DB_SUCCESS) {
+				mem_heap_free(heap);
+
+				return(err);
+			}
 		}
 
 		node->index = dict_table_get_next_index(node->index);
@@ -657,7 +725,7 @@ row_undo_mod(
 	if (node->table == NULL) {
 		found = FALSE;
 	} else {
-	  	found = row_undo_search_clust_to_pcur(node, thr);
+		found = row_undo_search_clust_to_pcur(node);
 	}
 
 	if (!found) {
