@@ -96,6 +96,7 @@ trx_create(
 	trx->check_unique_secondary = TRUE;
 
 	trx->flush_log_later = FALSE;
+	trx->must_flush_log_later = FALSE;
 
 	trx->dict_operation = FALSE;
 
@@ -151,7 +152,7 @@ trx_create(
 	trx->n_tickets_to_enter_innodb = 0;
 
 	trx->auto_inc_lock = NULL;
-	trx->n_tables_locked = 0;
+	trx->n_lock_table_exp = 0;
 	
 	trx->read_view_heap = mem_heap_create(256);
 	trx->read_view = NULL;
@@ -279,7 +280,7 @@ trx_free(
 
 	ut_a(!trx->has_search_latch);
 	ut_a(!trx->auto_inc_lock);
-	ut_a(!trx->n_tables_locked);
+	ut_a(!trx->n_lock_table_exp);
 
 	ut_a(trx->dict_operation_lock_mode == 0);
 
@@ -634,6 +635,8 @@ trx_commit_off_kernel(
 #endif /* UNIV_SYNC_DEBUG */
 
 	rseg = trx->rseg;
+
+	trx->must_flush_log_later = FALSE;
 	
 	if (trx->insert_undo != NULL || trx->update_undo != NULL) {
 
@@ -798,6 +801,7 @@ trx_commit_off_kernel(
 
                 if (trx->flush_log_later) {
                         /* Do nothing yet */
+			trx->must_flush_log_later = TRUE;
                 } else if (srv_flush_log_at_trx_commit == 0) {
                         /* Do nothing */
                 } else if (srv_flush_log_at_trx_commit == 1) {
@@ -1516,7 +1520,9 @@ trx_commit_complete_for_mysql(
 	
 	trx->op_info = (char*)"flushing log";
 
-        if (srv_flush_log_at_trx_commit == 0) {
+	if (!trx->must_flush_log_later) {
+		/* Do nothing */
+	} if (srv_flush_log_at_trx_commit == 0) {
                 /* Do nothing */
         } else if (srv_flush_log_at_trx_commit == 1) {
                 if (srv_unix_file_flush_method == SRV_UNIX_NOSYNC) {
@@ -1538,6 +1544,8 @@ trx_commit_complete_for_mysql(
                 ut_error;
         }
 
+	trx->must_flush_log_later = FALSE;
+
 	trx->op_info = (char*)"";
 
         return(0);
