@@ -276,6 +276,8 @@ static struct my_option my_long_options[] =
 };
 
 
+#include <help_start.h>
+
 static void print_version(void)
 {
   printf("%s Ver 5.10 for %s on %s\n", my_progname, SYSTEM_TYPE, MACHINE_TYPE);
@@ -299,6 +301,8 @@ static void usage(void)
   my_print_variables(my_long_options);
 }
 
+#include <help_end.h>
+
 
 static my_bool
 get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
