@@ -293,9 +293,9 @@ void Item_func_interval::fix_length_and_dec()
   }
   maybe_null=0; max_length=2;
   used_tables_cache|=     item->used_tables();
-  not_null_tables_cache&= item->not_null_tables();
+  not_null_tables_cache=  item->not_null_tables();
   with_sum_func= with_sum_func || item->with_sum_func;
-  const_item_cache&=item->const_item();
+  const_item_cache&=	  item->const_item();
 }
 
 
@@ -463,10 +463,9 @@ static Item_result item_store_type(Item_result a,Item_result b)
 {
   if (a == STRING_RESULT || b == STRING_RESULT)
     return STRING_RESULT;
-  else if (a == REAL_RESULT || b == REAL_RESULT)
+  if (a == REAL_RESULT || b == REAL_RESULT)
     return REAL_RESULT;
-  else
-    return INT_RESULT;
+  return INT_RESULT;
 }
 
 void
@@ -536,8 +535,8 @@ Item_func_if::fix_length_and_dec()
   decimals=max(args[1]->decimals,args[2]->decimals);
   enum Item_result arg1_type=args[1]->result_type();
   enum Item_result arg2_type=args[2]->result_type();
-  bool null1=args[1]->null_value;
-  bool null2=args[2]->null_value;
+  bool null1=args[1]->const_item() && args[1]->null_value;
+  bool null2=args[2]->const_item() && args[2]->null_value;
 
   if (null1)
   {
@@ -617,7 +616,7 @@ double
 Item_func_nullif::val()
 {
   double value;
-  if (!(this->*cmp_func)() || null_value)
+  if (!(this->*cmp_func)())
   {
     null_value=1;
     return 0.0;
@@ -631,7 +630,7 @@ longlong
 Item_func_nullif::val_int()
 {
   longlong value;
-  if (!(this->*cmp_func)() || null_value)
+  if (!(this->*cmp_func)())
   {
     null_value=1;
     return 0;
@@ -645,7 +644,7 @@ String *
 Item_func_nullif::val_str(String *str)
 {
   String *res;
-  if (!(this->*cmp_func)() || null_value)
+  if (!(this->*cmp_func)())
   {
     null_value=1;
     return 0;
@@ -655,6 +654,15 @@ Item_func_nullif::val_str(String *str)
   return res;
 }
 
+
+bool
+Item_func_nullif::is_null()
+{
+  if (!(this->*cmp_func)())
+    return (null_value= 1);
+  return 0;
+}
+
 /*
   CASE expression 
   Return the matching ITEM or NULL if all compares (including else) failed
@@ -1079,7 +1087,8 @@ void Item_func_in::fix_length_and_dec()
   maybe_null= item->maybe_null;
   max_length=2;
   used_tables_cache|=     item->used_tables();
-  not_null_tables_cache&= item->not_null_tables();
+  /* not_null_tables_cache is only dependent on the argument to in */
+  not_null_tables_cache=  item->not_null_tables();
   const_item_cache&=      item->const_item();
 }
 
