@@ -684,7 +684,7 @@ void Item_func_round::fix_length_and_dec()
     if (tmp < 0)
       decimals=0;
     else
-      decimals=tmp;
+      decimals=min(tmp,NOT_FIXED_DEC);
   }
 }
 
@@ -1071,6 +1071,7 @@ static const char separator=',';
 
 longlong Item_func_find_in_set::val_int()
 {
+  bool binary_cmp= args[0]->binary || args[1]->binary;
   if (enum_value)
   {
     ulonglong tmp=(ulonglong) args[1]->val_int();
@@ -1103,12 +1104,25 @@ longlong Item_func_find_in_set::val_int()
     do
     {
       const char *pos= f_pos;
-      while (pos != f_end)
+      if (binary_cmp)
       {
-	if (toupper(*str) != toupper(*pos))
-	  goto not_found;
-	str++;
-	pos++;
+        while (pos != f_end)
+        {
+          if (*str != *pos)
+            goto not_found;
+          str++;
+          pos++;
+        }
+      }
+      else
+      {
+        while (pos != f_end)
+        {
+          if (toupper(*str) != toupper(*pos))
+            goto not_found;
+          str++;
+          pos++;
+        }
       }
       if (str == real_end || str[0] == separator)
 	return (longlong) position;
@@ -1124,11 +1138,8 @@ longlong Item_func_find_in_set::val_int()
 longlong Item_func_bit_count::val_int()
 {
   ulonglong value= (ulonglong) args[0]->val_int();
-  if (args[0]->null_value)
-  {
-    null_value=1; /* purecov: inspected */
+  if ((null_value= args[0]->null_value))
     return 0; /* purecov: inspected */
-  }
   return (longlong) my_count_bits(value);
 }
 
@@ -1286,7 +1297,7 @@ udf_handler::fix_fields(THD *thd,TABLE_LIST *tables,Item_result_field *func,
     func->max_length=min(initid.max_length,MAX_BLOB_WIDTH);
     func->maybe_null=initid.maybe_null;
     const_item_cache=initid.const_item;
-    func->decimals=min(initid.decimals,31);
+    func->decimals=min(initid.decimals,NOT_FIXED_DEC);
   }
   initialized=1;
   if (error)
@@ -2243,6 +2254,11 @@ bool Item_func_match::fix_fields(THD *thd,struct st_table_list *tlist)
     key=NO_SUCH_KEY;
   const_item_cache=0;
   table=((Item_field *)fields.head())->field->table;
+  if (!(table->file->table_flags() & HA_CAN_FULLTEXT))
+  {
+    my_error(ER_TABLE_CANT_HANDLE_FULLTEXT, MYF(0));
+    return 1;
+  }
   table->fulltext_searched=1;
   record=table->record[0];
   if (key == NO_SUCH_KEY && mode != FT_BOOL)
