@@ -208,6 +208,7 @@ String *Item_func_aes_decrypt::val_str(String *str)
 void Item_func_aes_decrypt::fix_length_and_dec()
 {
    max_length=args[0]->max_length;
+   maybe_null= 1;
 }
 
 
@@ -1083,11 +1084,23 @@ String *Item_func_substr_index::val_str(String *str)
       }
     }
     else
-    {					// Start counting at end
-      for (offset=res->length() ; ; offset-=delimeter_length-1)
+    {
+      /*
+        Negative index, start counting at the end
+      */
+      for (offset=res->length(); offset ;)
       {
+        /* 
+          this call will result in finding the position pointing to one 
+          address space less than where the found substring is located
+          in res
+        */
 	if ((int) (offset=res->strrstr(*delimeter,offset)) < 0)
 	  return res;			// Didn't find, return org string
+        /*
+          At this point, we've searched for the substring
+          the number of times as supplied by the index value
+        */
 	if (!++count)
 	{
 	  offset+=delimeter_length;
@@ -1465,7 +1478,8 @@ Item_func_format::Item_func_format(Item *org,int dec) :Item_str_func(org)
 String *Item_func_format::val_str(String *str)
 {
   double nr	=args[0]->val();
-  uint32 diff,length,str_length;
+  uint32 length,str_length;
+  int diff;
   uint dec;
   if ((null_value=args[0]->null_value))
     return 0; /* purecov: inspected */
@@ -1483,17 +1497,17 @@ String *Item_func_format::val_str(String *str)
   if (str_length >= dec+4)
   {
     char *tmp,*pos;
-    length= str->length()+(diff=(str_length- dec-1)/3);
+    length= str->length()+(diff= (int)(str_length- dec-1)/3);
     str= copy_if_not_alloced(&tmp_str,str,length);
     str->length(length);
     tmp= (char*) str->ptr()+length - dec-1;
     for (pos= (char*) str->ptr()+length-1; pos != tmp; pos--)
-      pos[0]= pos[-(int) diff];
+      pos[0]= pos[-diff];
     while (diff)
     {
-      pos[0]=pos[-(int) diff]; pos--;
-      pos[0]=pos[-(int) diff]; pos--;
-      pos[0]=pos[-(int) diff]; pos--;
+      pos[0]=pos[-diff]; pos--;
+      pos[0]=pos[-diff]; pos--;
+      pos[0]=pos[-diff]; pos--;
       pos[0]=',';
       pos--;
       diff--;
@@ -2183,18 +2197,13 @@ String *Item_func_quote::val_str(String *str)
   for (from= (char*) arg->ptr(), end= from + arg_length; from < end; from++)
     new_length+= get_esc_bit(escmask, (uchar) *from);
 
-  /*
-    We have to use realloc() instead of alloc() as we want to keep the
-    old result in str
-  */
-  if (str->realloc(new_length))
+  if (tmp_value.alloc(new_length))
     goto null;
 
   /*
-    As 'arg' and 'str' may be the same string, we must replace characters
-    from the end to the beginning
+    We replace characters from the end to the beginning
   */
-  to= (char*) str->ptr() + new_length - 1;
+  to= (char*) tmp_value.ptr() + new_length - 1;
   *to--= '\'';
   for (start= (char*) arg->ptr(),end= start + arg_length; end-- != start; to--)
   {
@@ -2222,9 +2231,9 @@ String *Item_func_quote::val_str(String *str)
     }
   }
   *to= '\'';
-  str->length(new_length);
+  tmp_value.length(new_length);
   null_value= 0;
-  return str;
+  return &tmp_value;
 
 null:
   null_value= 1;
