@@ -4,6 +4,7 @@
 # Sligtly updated by Monty
 # Cleaned up again by Matt
 # Fixed by Sergei
+# List of failed cases (--force) backported from 4.1 by Joerg
 # :-)
 
 #++
@@ -200,12 +201,38 @@ MYSQL_MANAGER_PORT=9305 # needs to be out of the way of slaves
 MYSQL_MANAGER_PW_FILE=$MYSQL_TEST_DIR/var/tmp/manager.pwd
 MYSQL_MANAGER_LOG=$MYSQL_TEST_DIR/var/log/manager.log
 MYSQL_MANAGER_USER=root
+
+#
+# To make it easier for different devs to work on the same host,
+# an environment variable can be used to control all ports. A small
+# number is to be used, 0 - 16 or similar.
+#
+# Note the MASTER_MYPORT has to be set the same in all 4.x and 5.x
+# versions of this script, else a 4.0 test run might conflict with a
+# 5.1 test run, even if different MTR_BUILD_THREAD is used. This means
+# all port numbers might not be used in this version of the script.
+#
+if [ -n "$MTR_BUILD_THREAD" ] ; then
+  MASTER_MYPORT=`expr $MTR_BUILD_THREAD '*' 10 + 10000`
+  MYSQL_MANAGER_PORT=`expr $MASTER_MYPORT + 2`
+  SLAVE_MYPORT=`expr $MASTER_MYPORT + 3`
+
+  echo "Using MTR_BUILD_THREAD   = $MTR_BUILD_THREAD"
+  echo "Using MASTER_MYPORT      = $MASTER_MYPORT"
+  echo "Using MYSQL_MANAGER_PORT = $MYSQL_MANAGER_PORT"
+  echo "Using SLAVE_MYPORT       = $SLAVE_MYPORT"
+fi
+
 NO_SLAVE=0
 USER_TEST=
+FAILED_CASES=
 
 EXTRA_MASTER_OPT=""
 EXTRA_MYSQL_TEST_OPT=""
 USE_RUNNING_SERVER=1
+# backport from 5.1, disabled - this substitution is not done in 4.0
+# USE_NDBCLUSTER=@USE_NDBCLUSTER@
+# USE_NDBCLUSTER_ONLY=0
 DO_GCOV=""
 DO_GDB=""
 MANUAL_GDB=""
@@ -223,6 +250,8 @@ START_WAIT_TIMEOUT=10
 STOP_WAIT_TIMEOUT=10
 MYSQL_TEST_SSL_OPTS=""
 
+$ECHO "Logging: $0 $*"   # To ensure we see all arguments in the output, for the test analysis tool
+
 while test $# -gt 0; do
   case "$1" in
     --user=*) DBUSER=`$ECHO "$1" | $SED -e "s;--user=;;"` ;;
@@ -235,6 +264,14 @@ while test $# -gt 0; do
       SLAVE_MYSQLD=`$ECHO "$1" | $SED -e "s;--slave-binary=;;"` ;;
     --local)   USE_RUNNING_SERVER="" ;;
     --extern)   USE_RUNNING_SERVER="1" ;;
+    --with-ndbcluster)
+#     USE_NDBCLUSTER="--ndbcluster" ;;
+      $ECHO "Option '--with-ndbcluster' is ignored in this version" ;;
+    --with-ndbcluster-only)
+#     USE_NDBCLUSTER="--ndbcluster"
+#     USE_NDBCLUSTER_SLAVE="--ndbcluster"
+#     USE_NDBCLUSTER_ONLY=1 ;;
+      $ECHO "Option '--with-ndbcluster-only' is ignored in this version" ;;
     --tmpdir=*) MYSQL_TMP_DIR=`$ECHO "$1" | $SED -e "s;--tmpdir=;;"` ;;
     --local-master)
       MASTER_MYPORT=3306;
@@ -349,9 +386,11 @@ while test $# -gt 0; do
       VALGRIND=`which valgrind` # this will print an error if not found
       # Give good warning to the user and stop
       if [ -z "$VALGRIND" ] ; then
-        $ECHO "You need to have the 'valgrind' program in your PATH to run mysql-test-run with option --valgrind. Valgrind's home page is http://developer.kde.org/~sewardj ."
+        $ECHO "You need to have the 'valgrind' program in your PATH to run mysql-test-run with option --valgrind. Valgrind's home page is http://valgrind.kde.org ."
         exit 1
       fi
+      # >=2.1.2 requires the --tool option, some versions write to stdout, some to stderr
+      valgrind --help 2>&1 | grep "\-\-tool" > /dev/null && VALGRIND="$VALGRIND --tool=memcheck"
       VALGRIND="$VALGRIND --alignment=8 --leak-check=yes --num-callers=16"
       EXTRA_MASTER_MYSQLD_OPT="$EXTRA_MASTER_MYSQLD_OPT --skip-safemalloc --skip-bdb"
       EXTRA_SLAVE_MYSQLD_OPT="$EXTRA_SLAVE_MYSQLD_OPT --skip-safemalloc --skip-bdb"
@@ -384,6 +423,13 @@ while test $# -gt 0; do
     --fast)
       FAST_START=1
       ;;
+    --comment=*)
+      TMP=`$ECHO "$1" | $SED -e "s;--comment=;;"`
+      echo
+      echo '############################################'
+      echo "# $TMP"
+      echo '############################################'
+      ;;
     -- )  shift; break ;;
     --* ) $ECHO "Unrecognized option: $1"; exit 1 ;;
     * ) break ;;
@@ -589,6 +635,8 @@ show_failed_diff ()
     echo "Please follow the instructions outlined at"
     echo "http://www.mysql.com/doc/en/Reporting_mysqltest_bugs.html"
     echo "to find the reason to this problem and how to report this."
+    echo ""
+    echo "Test $1 failed!"
   fi
 }
 
@@ -668,7 +716,7 @@ report_stats () {
     #
     $RM -f $MY_LOG_DIR/warnings $MY_LOG_DIR/warnings.tmp
     # Remove some non fatal warnings from the log files
-    $SED -e 's!Warning:  Table:.* on delete!!g' \
+    $SED -e 's!Warning:  Table:.* on delete!!g' -e 's!Warning: Setting lower_case_table_names=2!!g' -e 's!Warning: One can only use the --user.*root!!g' \
          $MY_LOG_DIR/*.err > $MY_LOG_DIR/warnings.tmp
 
     found_error=0
@@ -1329,7 +1377,7 @@ run_testcase ()
 	show_failed_diff $result_file
 	$ECHO
 	if [ x$FORCE != x1 ] ; then
-	 $ECHO "Aborting. To continue, re-run with '--force'."
+	 $ECHO "Aborting: $tname failed. To continue, re-run with '--force'."
 	 $ECHO
          if [ -z "$DO_GDB" ] && [ -z "$USE_RUNNING_SERVER" ] && [ -z "$DO_DDD" ]
 	 then
@@ -1338,7 +1386,7 @@ run_testcase ()
    	 fi
 	 exit 1
 	fi
-
+	FAILED_CASES="$FAILED_CASES $tname"
         if [ -z "$DO_GDB" ] && [ -z "$USE_RUNNING_SERVER" ] && [ -z "$DO_DDD" ]
 	then
 	  mysql_restart
@@ -1481,4 +1529,10 @@ $ECHO
 [ "$DO_GCOV" ] && gcov_collect # collect coverage information
 [ "$DO_GPROF" ] && gprof_collect # collect coverage information
 
-exit 0
+if [ $TOT_FAIL -ne 0 ]; then
+  $ECHO "mysql-test-run: *** Failing the test(s):$FAILED_CASES"
+  $ECHO
+  exit 1
+else
+  exit 0
+fi
