@@ -120,6 +120,20 @@ void init_thr_alarm(uint max_alarms)
   DBUG_VOID_RETURN;
 }
 
+
+void resize_thr_alarm(uint max_alarms)
+{
+  pthread_mutex_lock(&LOCK_alarm);
+  /*
+    It's ok not to shrink the queue as there may be more pending alarms than
+    than max_alarms
+  */
+  if (alarm_queue.elements < max_alarms)
+    resize_queue(&alarm_queue,max_alarms+1);
+  pthread_mutex_unlock(&LOCK_alarm);
+}
+
+
 /*
   Request alarm after sec seconds.
 
@@ -700,6 +714,9 @@ void thr_alarm_info(ALARM_INFO *info)
   bzero((char*) info, sizeof(*info));
 }
 
+void resize_thr_alarm(uint max_alarms)
+{
+}
 
 /*****************************************************************************
   thr_alarm for win95
@@ -753,7 +770,9 @@ bool thr_got_alarm(thr_alarm_t *alrm_ptr)
 void thr_end_alarm(thr_alarm_t *alrm_ptr)
 {
   thr_alarm_t alrm= *alrm_ptr;
+  /* alrm may be zero if thr_alarm aborted with an error */
   if (alrm && alrm->crono)
+
   {
     KillTimer(NULL, alrm->crono);
     alrm->crono = 0;
@@ -779,6 +798,10 @@ void thr_alarm_info(ALARM_INFO *info)
   bzero((char*) info, sizeof(*info));
 }
 
+void resize_thr_alarm(uint max_alarms)
+{
+}
+
 #endif /* __WIN__ */
 
 #endif /* THREAD */
