@@ -15,6 +15,7 @@ MV="mv"
 STRIP=1
 DEBUG=0
 SILENT=0
+MACHINE=
 TMP=/tmp
 SUFFIX=""
 
@@ -25,6 +26,7 @@ parse_arguments() {
       --tmp=*)    TMP=`echo "$arg" | sed -e "s;--tmp=;;"` ;;
       --suffix=*) SUFFIX=`echo "$arg" | sed -e "s;--suffix=;;"` ;;
       --no-strip) STRIP=0 ;;
+      --machine=*)  MACHINE=`echo "$arg" | sed -e "s;--machine=;;"` ;;
       --silent)   SILENT=1 ;;
       *)
 	echo "Unknown argument '$arg'"
@@ -74,9 +76,6 @@ if [ $BASE_SYSTEM != "netware" ] ; then
 fi
 
 for i in ChangeLog \
-         Docs/manual.html \
-         Docs/manual.txt \
-         Docs/manual_toc.html \
 				 Docs/mysql.info
 do
   if [ -f $i ]
@@ -255,8 +254,17 @@ if [ -d $BASE/sql-bench/SCCS ] ; then
   find $BASE/sql-bench -name SCCS -print | xargs rm -r -f
 fi
 
+# Use the override --machine if present
+if [ -n "$MACHINE" ] ; then
+  machine=$MACHINE
+fi
+
 # Change the distribution to a long descriptive name
 NEW_NAME=mysql@MYSQL_SERVER_SUFFIX@-$version-$system-$machine$SUFFIX
+
+# Print the platform name for build logs
+echo "PLATFORM NAME: $system-$machine"
+
 BASE2=$TMP/$NEW_NAME
 rm -r -f $BASE2
 mv $BASE $BASE2
@@ -271,7 +279,7 @@ then
   gcclib=`@CC@ --print-libgcc-file`
   if test $? -ne 0
   then
-    print "Warning: Couldn't find libgcc.a!"
+    echo "Warning: Couldn't find libgcc.a!"
   else
     $CP $gcclib libmygcc.a
   fi
