@@ -178,18 +178,8 @@ rm -r -f "$BASE/share/Makefile"
 rm -r -f "$BASE/share/Makefile.in"
 rm -r -f "$BASE/share/Makefile.am"
 
-#
-# Clean up if we did this from a bk tree
-#
-
-if [ -d $BASE/SCCS ]
-then
-  find $BASE/ -type d -name SCCS -printf " \"%p\"" | xargs rm -r -f
-fi
-
 mkdir $BASE/Docs $BASE/extra $BASE/include
 
-
 #
 # Copy directory files
 #
@@ -282,12 +272,10 @@ touch $BASE/innobase/ib_config.h
 #
 
 cd $SOURCE
-for i in COPYING ChangeLog README \
+for i in COPYING ChangeLog README EXCEPTIONS-CLIENT\
          INSTALL-SOURCE INSTALL-WIN \
          INSTALL-WIN-SOURCE \
-         Docs/manual_toc.html  Docs/manual.html \
-         Docs/manual.txt Docs/mysqld_error.txt \
-         Docs/INSTALL-BINARY
+         Docs/INSTALL-BINARY Docs/manual.chm
 
 do
   print_debug "Copying file '$i'"
@@ -297,12 +285,17 @@ do
   fi
 done
 
+#
+# support files
+#
+mkdir $BASE/support-files
+cp support-files/*.cnf $BASE/support-files
+
 #
 # Raw dirs from source tree
 #
 
-for i in Docs/Flags scripts sql-bench SSL \
-         tests
+for i in scripts sql-bench SSL tests
 do
   print_debug "Copying directory '$i'"
   if [ -d $i ]
@@ -312,7 +305,7 @@ do
 done
 
 #
-# Fix some windows files
+# Fix some windows files to avoid compiler warnings
 #
 
 ./extra/replace std:: "" -- $BASE/sql/sql_yacc.cpp
@@ -320,6 +313,15 @@ done
 unix_to_dos $BASE/README
 mv $BASE/README $BASE/README.txt
 
+#
+# Clean up if we did this from a bk tree
+#
+
+if [ -d $BASE/SSL/SCCS ]
+then
+  find $BASE/ -type d -name SCCS -printf " \"%p\"" | xargs rm -r -f
+fi
+
 #
 # Initialize the initial data directory
 #
@@ -327,6 +329,10 @@ mv $BASE/README $BASE/README.txt
 if [ -f scripts/mysql_install_db ]; then
   print_debug "Initializing the 'data' directory"
   scripts/mysql_install_db --no-defaults --windows --datadir=$BASE/data
+  if test "$?" = 1
+  then
+    exit 1;
+  fi
 fi
 
 #
