@@ -240,7 +240,7 @@ int berkeley_show_logs(THD *thd)
   MEM_ROOT *old_root=my_pthread_getspecific_ptr(MEM_ROOT*,THR_MALLOC);
   DBUG_ENTER("berkeley_show_logs");
 
-  init_alloc_root(&show_logs_root, 1024, 1024);
+  init_sql_alloc(&show_logs_root, 1024, 1024);
   my_pthread_setspecific_ptr(THR_MALLOC,&show_logs_root);
 
   if ((error= log_archive(db_env, &all_logs, DB_ARCH_ABS | DB_ARCH_LOG,
@@ -336,8 +336,9 @@ void berkeley_cleanup_log_files(void)
 ** Berkeley DB tables
 *****************************************************************************/
 
+static const char *bdb_bas_exts[]= { ha_berkeley_ext, NullS };
 const char **ha_berkeley::bas_ext() const
-{ static const char *ext[]= { ha_berkeley_ext, NullS }; return ext; }
+{ return bdb_bas_exts; }
 
 
 static int
@@ -358,7 +359,7 @@ berkeley_cmp_packed_key(DB *file, const DBT *new_key, const DBT *saved_key)
   KEY_PART_INFO *key_part= key->key_part, *end=key_part+key->key_parts;
   uint key_length=new_key->size;
 
-  for ( ; key_part != end && (int) key_length > 0; key_part++)
+  for (; key_part != end && (int) key_length > 0; key_part++)
   {
     int cmp;
     if (key_part->null_bit)
@@ -396,7 +397,7 @@ berkeley_cmp_fix_length_key(DB *file, const DBT *new_key, const DBT *saved_key)
   KEY_PART_INFO *key_part= key->key_part, *end=key_part+key->key_parts;
   uint key_length=new_key->size;
 
-  for ( ; key_part != end && (int) key_length > 0 ; key_part++)
+  for (; key_part != end && (int) key_length > 0 ; key_part++)
   {
     int cmp;
     if ((cmp=key_part->field->pack_cmp(new_key_ptr,saved_key_ptr,0)))
@@ -417,7 +418,7 @@ berkeley_key_cmp(TABLE *table, KEY *key_info, const char *key, uint key_length)
   KEY_PART_INFO *key_part= key_info->key_part,
 		*end=key_part+key_info->key_parts;
 
-  for ( ; key_part != end && (int) key_length > 0; key_part++)
+  for (; key_part != end && (int) key_length > 0; key_part++)
   {
     int cmp;
     if (key_part->null_bit)
@@ -544,7 +545,10 @@ int ha_berkeley::open(const char *name, int mode, uint test_if_locked)
 	(*ptr)->set_bt_compare(*ptr, berkeley_cmp_packed_key);
 	(*ptr)->app_private= (void*) (table->key_info+i);
 	if (!(table->key_info[i].flags & HA_NOSAME))
+	{
+	  DBUG_PRINT("bdb",("Setting DB_DUP for key %u", i));
 	  (*ptr)->set_flags(*ptr, DB_DUP);
+	}
 	if ((error=((*ptr)->open(*ptr, name_buff, part, DB_BTREE,
 				 open_mode, 0))))
 	{
@@ -561,7 +565,7 @@ int ha_berkeley::open(const char *name, int mode, uint test_if_locked)
       ref_length=0;
       KEY_PART_INFO *key_part= table->key_info[primary_key].key_part;
       KEY_PART_INFO *end=key_part+table->key_info[primary_key].key_parts;
-      for ( ; key_part != end ; key_part++)
+      for (; key_part != end ; key_part++)
 	ref_length+= key_part->field->max_packed_col_length(key_part->length);
       share->fixed_length_primary_key=
 	(ref_length == table->key_info[primary_key].key_length);
@@ -701,7 +705,7 @@ void ha_berkeley::unpack_key(char *record, DBT *key, uint index)
 		*end=key_part+key_info->key_parts;
 
   char *pos=(char*) key->data;
-  for ( ; key_part != end; key_part++)
+  for (; key_part != end; key_part++)
   {
     if (key_part->null_bit)
     {
@@ -717,8 +721,8 @@ void ha_berkeley::unpack_key(char *record, DBT *key, uint index)
       }
       record[key_part->null_offset]&= ~key_part->null_bit;
     }
-    pos= (char*) key_part->field->unpack(record + key_part->field->offset(),
-					 pos);
+    pos= (char*) key_part->field->unpack_key(record + key_part->field->offset(),
+                                             pos, key_part->length);
   }
 }
 
@@ -747,7 +751,7 @@ DBT *ha_berkeley::create_key(DBT *key, uint keynr, char *buff,
 
   key->data=buff;
   key->app_private= key_info;
-  for ( ; key_part != end && key_length > 0; key_part++)
+  for (; key_part != end && key_length > 0; key_part++)
   {
     if (key_part->null_bit)
     {
@@ -925,7 +929,7 @@ int ha_berkeley::key_cmp(uint keynr, const byte * old_row,
   KEY_PART_INFO *key_part=table->key_info[keynr].key_part;
   KEY_PART_INFO *end=key_part+table->key_info[keynr].key_parts;
 
-  for ( ; key_part != end ; key_part++)
+  for (; key_part != end ; key_part++)
   {
     if (key_part->null_bit)
     {
@@ -1043,9 +1047,9 @@ int ha_berkeley::restore_keys(DB_TXN *trans, key_map changed_keys,
 	break; /* purecov: inspected */
     }
   }
-  
+
 err:
-  dbug_assert(error != DB_KEYEXIST);
+  DBUG_ASSERT(error != DB_KEYEXIST);
   DBUG_RETURN(error);
 }
 
@@ -1187,7 +1191,7 @@ int ha_berkeley::remove_key(DB_TXN *trans, uint keynr, const byte *record,
 	   ((table->key_info[keynr].flags & (HA_NOSAME | HA_NULL_PART_KEY)) ==
 	    HA_NOSAME))
   {						// Unique key
-    dbug_assert(keynr == primary_key || prim_key->data != key_buff2);
+    DBUG_ASSERT(keynr == primary_key || prim_key->data != key_buff2);
     error=key_file[keynr]->del(key_file[keynr], trans,
 			       keynr == primary_key ?
 			       prim_key :
@@ -1201,7 +1205,7 @@ int ha_berkeley::remove_key(DB_TXN *trans, uint keynr, const byte *record,
       row to find the key to be delete and delete it.
       We will never come here with keynr = primary_key
     */
-    dbug_assert(keynr != primary_key && prim_key->data != key_buff2);
+    DBUG_ASSERT(keynr != primary_key && prim_key->data != key_buff2);
     DBC *tmp_cursor;
     if (!(error=key_file[keynr]->cursor(key_file[keynr], trans,
 					&tmp_cursor, 0)))
@@ -1331,7 +1335,7 @@ int ha_berkeley::index_init(uint keynr)
 int ha_berkeley::index_end()
 {
   int error=0;
-  DBUG_ENTER("index_end");
+  DBUG_ENTER("ha_berkely::index_end");
   if (cursor)
   {
     DBUG_PRINT("enter",("table: '%s'", table->real_name));
@@ -1420,6 +1424,8 @@ int ha_berkeley::index_read(byte * buf, const byte * key,
   bzero((char*) &row,sizeof(row));
   if (key_len == key_info->key_length)
   {
+    if (find_flag == HA_READ_AFTER_KEY)
+      key_info->handler.bdb_return_if_eq= 1;
     error=read_row(cursor->c_get(cursor, pack_key(&last_key,
 						  active_index,
 						  key_buff,
@@ -1428,6 +1434,7 @@ int ha_berkeley::index_read(byte * buf, const byte * key,
 				 (find_flag == HA_READ_KEY_EXACT ?
 				  DB_SET : DB_SET_RANGE)),
 		   (char*) buf, active_index, &row, (DBT*) 0, 0);
+    key_info->handler.bdb_return_if_eq= 0;
   }
   else
   {
@@ -1454,6 +1461,37 @@ int ha_berkeley::index_read(byte * buf, const byte * key,
   DBUG_RETURN(error);
 }
 
+/*
+  Read last key is solved by reading the next key and then reading
+  the previous key
+*/
+
+int ha_berkeley::index_read_last(byte * buf, const byte * key, uint key_len)
+{
+  DBT row;
+  int error;
+  KEY *key_info= &table->key_info[active_index];
+  DBUG_ENTER("ha_berkeley::index_read");
+
+  statistic_increment(ha_read_key_count,&LOCK_status);
+  bzero((char*) &row,sizeof(row));
+
+  /* read of partial key */
+  pack_key(&last_key, active_index, key_buff, key, key_len);
+  /* Store for compare */
+  memcpy(key_buff2, key_buff, (key_len=last_key.size));
+  key_info->handler.bdb_return_if_eq= 1;
+  error=read_row(cursor->c_get(cursor, &last_key, &row, DB_SET_RANGE),
+		 (char*) buf, active_index, &row, (DBT*) 0, 0);
+  key_info->handler.bdb_return_if_eq= 0;
+  bzero((char*) &row,sizeof(row));
+  if (read_row(cursor->c_get(cursor, &last_key, &row, DB_PREV),
+	       (char*) buf, active_index, &row, &last_key, 1) ||
+      berkeley_key_cmp(table, key_info, key_buff2, key_len))
+    error=HA_ERR_KEY_NOT_FOUND;
+  DBUG_RETURN(error);
+}
+
 
 int ha_berkeley::index_next(byte * buf)
 {
@@ -1553,7 +1591,7 @@ DBT *ha_berkeley::get_pos(DBT *to, byte *pos)
     KEY_PART_INFO *key_part=table->key_info[primary_key].key_part;
     KEY_PART_INFO *end=key_part+table->key_info[primary_key].key_parts;
 
-    for ( ; key_part != end ; key_part++)
+    for (; key_part != end ; key_part++)
       pos+=key_part->field->packed_col_length((char*) pos,key_part->length);
     to->size= (uint) (pos- (byte*) to->data);
   }
@@ -1565,21 +1603,53 @@ int ha_berkeley::rnd_pos(byte * buf, byte *pos)
 {
   DBT db_pos;
   statistic_increment(ha_read_rnd_count,&LOCK_status);
+  DBUG_ENTER("ha_berkeley::rnd_pos");
 
   active_index= (uint) -1;			// Don't delete via cursor
-  return read_row(file->get(file, transaction,
-			    get_pos(&db_pos, pos),
-			    &current_row, 0),
-		 (char*) buf, primary_key, &current_row, (DBT*) 0, 0);
+  DBUG_RETURN(read_row(file->get(file, transaction,
+				 get_pos(&db_pos, pos),
+				 &current_row, 0),
+		       (char*) buf, primary_key, &current_row, (DBT*) 0, 0));
 }
 
+/*
+  Set a reference to the current record in (ref,ref_length).
+
+  SYNOPSIS
+    ha_berkeley::position()
+    record                      The current record buffer
+
+  DESCRIPTION
+    The BDB handler stores the primary key in (ref,ref_length).
+    There is either an explicit primary key, or an implicit (hidden)
+    primary key.
+    During open(), 'ref_length' is calculated as the maximum primary
+    key length. When an actual key is shorter than that, the rest of
+    the buffer must be cleared out. The row cannot be identified, if
+    garbage follows behind the end of the key. There is no length
+    field for the current key, so that the whole ref_length is used
+    for comparison.
+
+  RETURN
+    nothing
+*/
+
 void ha_berkeley::position(const byte *record)
 {
   DBT key;
+  DBUG_ENTER("ha_berkeley::position");
   if (hidden_primary_key)
+  {
+    DBUG_ASSERT(ref_length == BDB_HIDDEN_PRIMARY_KEY_LENGTH);
     memcpy_fixed(ref, (char*) current_ident, BDB_HIDDEN_PRIMARY_KEY_LENGTH);
+  }
   else
+  {
     create_key(&key, primary_key, (char*) ref, record);
+    if (key.size < ref_length)
+      bzero(ref + key.size, ref_length - key.size);
+  }
+  DBUG_VOID_RETURN;
 }
 
 
@@ -1600,8 +1670,9 @@ void ha_berkeley::info(uint flag)
 	share->rec_per_key[i];
     }
   }
-  else if (flag & HA_STATUS_ERRKEY)
-    errkey=last_dup_key;
+  /* Don't return key if we got an error for the internal primary key */
+  if (flag & HA_STATUS_ERRKEY && last_dup_key < table->keys)
+    errkey= last_dup_key;
   DBUG_VOID_RETURN;
 }
 
@@ -1669,7 +1740,7 @@ int ha_berkeley::external_lock(THD *thd, int lock_type)
       DBUG_ASSERT(thd->transaction.stmt.bdb_tid == 0);
       transaction=0;				// Safety
       /* First table lock, start transaction */
-      if ((thd->options & (OPTION_NOT_AUTO_COMMIT | OPTION_BEGIN |
+      if ((thd->options & (OPTION_NOT_AUTOCOMMIT | OPTION_BEGIN |
 			   OPTION_TABLE_LOCK)) &&
 	  !thd->transaction.all.bdb_tid)
       {
@@ -1799,7 +1870,7 @@ static int create_sub_table(const char *table_name, const char *sub_name,
   int error;
   DB *file;
   DBUG_ENTER("create_sub_table");
-  DBUG_PRINT("enter",("sub_name: %s",sub_name));
+  DBUG_PRINT("enter",("sub_name: %s  flags: %d",sub_name, flags));
 
   if (!(error=db_create(&file, db_env, 0)))
   {
@@ -1831,14 +1902,14 @@ int ha_berkeley::create(const char *name, register TABLE *form,
   char name_buff[FN_REFLEN];
   char part[7];
   uint index=1;
-  int error=1;
+  int error;
   DBUG_ENTER("ha_berkeley::create");
 
   fn_format(name_buff,name,"", ha_berkeley_ext,2 | 4);
 
   /* Create the main table that will hold the real rows */
-  if (create_sub_table(name_buff,"main",DB_BTREE,0))
-    DBUG_RETURN(1); /* purecov: inspected */
+  if ((error= create_sub_table(name_buff,"main",DB_BTREE,0)))
+    DBUG_RETURN(error); /* purecov: inspected */
 
   primary_key=table->primary_key;
   /* Create the keys */
@@ -1847,10 +1918,10 @@ int ha_berkeley::create(const char *name, register TABLE *form,
     if (i != primary_key)
     {
       sprintf(part,"key%02d",index++);
-      if (create_sub_table(name_buff, part, DB_BTREE,
-			   (table->key_info[i].flags & HA_NOSAME) ? 0 :
-			   DB_DUP))
-	DBUG_RETURN(1); /* purecov: inspected */
+      if ((error= create_sub_table(name_buff, part, DB_BTREE,
+				   (table->key_info[i].flags & HA_NOSAME) ? 0 :
+				   DB_DUP)))
+	DBUG_RETURN(error); /* purecov: inspected */
     }
   }
 
@@ -1858,16 +1929,15 @@ int ha_berkeley::create(const char *name, register TABLE *form,
   /* Is DB_BTREE the best option here ? (QUEUE can't be used in sub tables) */
 
   DB *status_block;
-  if (!db_create(&status_block, db_env, 0))
+  if (!(error=(db_create(&status_block, db_env, 0))))
   {
-    if (!status_block->open(status_block, name_buff,
-			    "status", DB_BTREE, DB_CREATE, 0))
+    if (!(error=(status_block->open(status_block, name_buff,
+				    "status", DB_BTREE, DB_CREATE, 0))))
     {
       char rec_buff[4+MAX_KEY*4];
       uint length= 4+ table->keys*4;
       bzero(rec_buff, length);
-      if (!write_status(status_block, rec_buff, length))
-	error=0;
+      error= write_status(status_block, rec_buff, length);
       status_block->close(status_block,0);
     }
   }
@@ -1875,19 +1945,41 @@ int ha_berkeley::create(const char *name, register TABLE *form,
 }
 
 
+
 int ha_berkeley::delete_table(const char *name)
 {
   int error;
   char name_buff[FN_REFLEN];
+  DBUG_ENTER("delete_table");
   if ((error=db_create(&file, db_env, 0)))
     my_errno=error; /* purecov: inspected */
   else
     error=file->remove(file,fn_format(name_buff,name,"",ha_berkeley_ext,2 | 4),
 		       NULL,0);
   file=0;					// Safety
+  DBUG_RETURN(error);
+}
+
+int ha_berkeley::rename_table(const char * from, const char * to)
+{
+  int error;
+  char from_buff[FN_REFLEN];
+  char to_buff[FN_REFLEN];
+
+  if ((error= db_create(&file, db_env, 0)))
+    my_errno= error;
+  else
+  {
+    /* On should not do a file->close() after rename returns */
+    error= file->rename(file, 
+			fn_format(from_buff, from, "", ha_berkeley_ext, 2 | 4),
+			NULL, fn_format(to_buff, to, "", ha_berkeley_ext,
+					2 | 4), 0);
+  }
   return error;
 }
 
+
 /*
   How many seeks it will take to read through the table
   This is to be comparable to the number returned by records_in_range so
@@ -1896,7 +1988,7 @@ int ha_berkeley::delete_table(const char *name)
 
 double ha_berkeley::scan_time()
 {
-  return records/3;
+  return rows2double(records/3);
 }
 
 ha_rows ha_berkeley::records_in_range(int keynr,
@@ -2039,6 +2131,53 @@ int ha_berkeley::analyze(THD* thd, HA_CHECK_OPT* check_opt)
 {
   DB_BTREE_STAT *stat=0;
   uint i;
+  DB_TXN_STAT *txn_stat_ptr= 0;
+
+  /*
+   Original bdb documentation says:
+   "The DB->stat method cannot be transaction-protected.
+   For this reason, it should be called in a thread of
+   control that has no open cursors or active transactions."
+   So, let's check if there are any changes have been done since
+   the beginning of the transaction..
+  */
+
+  /*
+    If it's a merge conflict here (4.0->4.1), please ignore it!
+
+    The reason of the conflict is the difference between versions of bdb:
+    mysql-4.0 uses bdb 3.2.9
+    mysql-4.1 uses bdb 4.1.24
+    Older one has global functions txn_stat and txn_id but
+    newer one has DB_ENV->txn_stat and DB_TXN->id
+  */
+  if (!txn_stat(db_env, &txn_stat_ptr, 0) &&
+      txn_stat_ptr && txn_stat_ptr->st_nactive>=2)
+  {
+    DB_TXN_ACTIVE *atxn_stmt= 0, *atxn_all= 0;
+    
+    DB_TXN *txn_all= (DB_TXN*) thd->transaction.all.bdb_tid;
+    u_int32_t all_id= txn_id(txn_all);
+    
+    DB_TXN *txn_stmt= (DB_TXN*) thd->transaction.stmt.bdb_tid;
+    u_int32_t stmt_id= txn_id(txn_stmt);
+    
+    DB_TXN_ACTIVE *cur= txn_stat_ptr->st_txnarray;
+    DB_TXN_ACTIVE *end= cur + txn_stat_ptr->st_nactive;
+    for (; cur!=end && (!atxn_stmt || !atxn_all); cur++)
+    {
+      if (cur->txnid==all_id) atxn_all= cur;
+      if (cur->txnid==stmt_id) atxn_stmt= cur;
+    }
+    
+    if (atxn_stmt && atxn_all &&
+	log_compare(&atxn_stmt->lsn,&atxn_all->lsn))
+    {
+      free(txn_stat_ptr);
+      return HA_ADMIN_REJECT;
+    }
+    free(txn_stat_ptr);
+  }
 
   for (i=0 ; i < table->keys ; i++)
   {
@@ -2047,7 +2186,7 @@ int ha_berkeley::analyze(THD* thd, HA_CHECK_OPT* check_opt)
       free(stat);
       stat=0;
     }
-    if (key_file[i]->stat(key_file[i], (void*) &stat, 0, 0))
+    if ((key_file[i]->stat)(key_file[i], (void*) &stat, 0, 0))
       goto err; /* purecov: inspected */
     share->rec_per_key[i]= (stat->bt_ndata /
 			    (stat->bt_nkeys ? stat->bt_nkeys : 1));
@@ -2060,7 +2199,7 @@ int ha_berkeley::analyze(THD* thd, HA_CHECK_OPT* check_opt)
       free(stat);
       stat=0;
     }
-    if (file->stat(file, (void*) &stat, 0, 0))
+    if ((file->stat)(file, (void*) &stat, 0, 0))
       goto err; /* purecov: inspected */
   }
   pthread_mutex_lock(&share->mutex);
@@ -2171,11 +2310,11 @@ static BDB_SHARE *get_share(const char *table_name, TABLE *table)
   if (!(share=(BDB_SHARE*) hash_search(&bdb_open_tables, (byte*) table_name,
 				       length)))
   {
-    ha_rows *rec_per_key;
+    ulong *rec_per_key;
     char *tmp_name;
     DB **key_file;
     u_int32_t *key_type;
-    
+
     if ((share=(BDB_SHARE *)
 	 my_multi_malloc(MYF(MY_WME | MY_ZEROFILL),
 			 &share, sizeof(*share),
