@@ -15,11 +15,11 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /*
- mini MySQL client to be included into the server to do server to server
- commincation by Sasha Pachev
+  mini MySQL client to be included into the server to do server to server
+  commincation by Sasha Pachev
 
- Note: all file-global symbols must begin with mc_ , even the static ones, just
- in case we decide to make them external at some point
+  Note: all file-global symbols must begin with mc_ , even the static ones,
+  just in case we decide to make them external at some point
 */
 
 #include <my_global.h>
@@ -117,7 +117,7 @@ HANDLE create_named_pipe(NET *net, uint connect_timeout, char **arg_host,
 			 char **arg_unix_socket)
 {
   HANDLE hPipe=INVALID_HANDLE_VALUE;
-  char szPipeName [ 257 ];
+  char pipe_name[512];
   DWORD dwMode;
   int i;
   my_bool testing_named_pipes=0;
@@ -126,13 +126,15 @@ HANDLE create_named_pipe(NET *net, uint connect_timeout, char **arg_host,
   if (!host || !strcmp(host,LOCAL_HOST))
     host=LOCAL_HOST_NAMEDPIPE;
 
-  sprintf( szPipeName, "\\\\%s\\pipe\\%s", host, unix_socket);
+  pipe_name[sizeof(pipe_name)-1]= 0;		/* Safety if too long string */
+  strxnmov(pipe_name, sizeof(pipe_name)-1, "\\\\", host, "\\pipe\\",
+	   unix_socket, NullS);
   DBUG_PRINT("info",("Server name: '%s'.  Named Pipe: %s",
 		     host, unix_socket));
 
   for (i=0 ; i < 100 ; i++)			/* Don't retry forever */
   {
-    if ((hPipe = CreateFile(szPipeName,
+    if ((hPipe = CreateFile(pipe_name,
 			    GENERIC_READ | GENERIC_WRITE,
 			    0,
 			    NULL,
@@ -148,7 +150,7 @@ HANDLE create_named_pipe(NET *net, uint connect_timeout, char **arg_host,
       return INVALID_HANDLE_VALUE;
     }
     /* wait for for an other instance */
-    if (! WaitNamedPipe(szPipeName, connect_timeout*1000) )
+    if (! WaitNamedPipe(pipe_name, connect_timeout*1000) )
     {
       net->last_errno=CR_NAMEDPIPEWAIT_ERROR;
       sprintf(net->last_error,ER(net->last_errno),host, unix_socket,
@@ -397,7 +399,7 @@ mc_net_safe_read(MYSQL *mysql)
   {
     DBUG_PRINT("error",("Wrong connection or packet. fd: %s  len: %d",
 			vio_description(net->vio),len));
-    if (socket_errno != SOCKET_EINTR)
+    if (!vio_was_interrupted(net->vio))
     {
       mc_end_server(mysql);
       if (net->last_errno != ER_NET_PACKET_TOO_LARGE)
@@ -547,7 +549,7 @@ mc_mysql_connect(MYSQL *mysql,const char *host, const char *user,
 {
   char		buff[NAME_LEN+USERNAME_LENGTH+100],*end,*host_info;
   my_socket	sock;
-  ulong		ip_addr;
+  in_addr_t	ip_addr;
   struct	sockaddr_in sock_addr;
   ulong		pkt_length;
   NET		*net= &mysql->net;
@@ -655,6 +657,11 @@ mc_mysql_connect(MYSQL *mysql,const char *host, const char *user,
     sprintf(host_info=buff,ER(CR_TCP_CONNECTION),host);
     DBUG_PRINT("info",("Server name: '%s'.  TCP sock: %d", host,port));
     thr_alarm_init(&alarmed);
+    /*
+      We don't have to check status for thr_alarm as it's not fatal if
+      we didn't manage to set an alarm. (In this case the socket call
+      will just block for a while).
+    */
     thr_alarm(&alarmed, net_read_timeout, &alarm_buff);
     sock = (my_socket) socket(AF_INET,SOCK_STREAM,0);
     thr_end_alarm(&alarmed);
