@@ -227,6 +227,8 @@ int
 my_net_write(NET *net,const char *packet,ulong len)
 {
   uchar buff[NET_HEADER_SIZE];
+  if (unlikely(!net->vio)) /* nowhere to write */
+    return 0;
   /*
     Big packets are handled by splitting them in packets of MAX_PACKET_LENGTH
     length. The last packet is always a packet that is < MAX_PACKET_LENGTH.
@@ -698,7 +700,7 @@ my_real_read(NET *net, ulong *complen)
 	  len= packet_error;
 	  net->error=2;				/* Close socket */
 #ifdef MYSQL_SERVER
-	  net->last_errno= (interrupted ? ER_NET_READ_INTERRUPTED :
+	  net->last_errno= (vio_was_interrupted(net->vio) ? ER_NET_READ_INTERRUPTED :
 			    ER_NET_READ_ERROR);
 #endif
 	  goto end;
@@ -833,7 +835,7 @@ my_net_read(NET *net)
 
     if (net->remain_in_buf)
     {
-      buf_length= net->buf_length;		// Data left in old packet
+      buf_length= net->buf_length;		/* Data left in old packet */
       first_packet_offset= start_of_packet= (net->buf_length -
 					     net->remain_in_buf);
       /* Restore the character that was overwritten by the end 0 */
