@@ -28,13 +28,14 @@ class FT_SELECT: public QUICK_SELECT {
 public:
   TABLE_REF *ref;
 
-  FT_SELECT(TABLE *table, TABLE_REF *tref) :
-      QUICK_SELECT (table,tref->key,1), ref(tref) { init(); }
+  FT_SELECT(THD *thd, TABLE *table, TABLE_REF *tref) :
+      QUICK_SELECT (thd, table, tref->key, 1), ref(tref) { init(); }
 
   int init() { return error=file->ft_init(); }
   int get_next() { return error=file->ft_read(record); }
 };
 
-QUICK_SELECT *get_ft_or_quick_select_for_ref(TABLE *table, JOIN_TAB *tab);
+QUICK_SELECT *get_ft_or_quick_select_for_ref(THD *thd, TABLE *table,
+					     JOIN_TAB *tab);
 
 #endif
