@@ -14,8 +14,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
-// Sasha Pachev <sasha@mysql.com> is currently in charge of this file
-
 #include "mysql_priv.h"
 #include "sql_repl.h"
 #include "sql_acl.h"
@@ -86,7 +84,7 @@ static int send_file(THD *thd)
   char fname[FN_REFLEN+1];
   const char *errmsg = 0;
   int old_timeout;
-  uint packet_len;
+  unsigned long packet_len;
   char buf[IO_SIZE];				// It's safe to alloc this
   DBUG_ENTER("send_file");
 
@@ -664,7 +662,8 @@ int start_slave(THD* thd , MASTER_INFO* mi,  bool net_report)
     thread_mask &= thd->lex.slave_thd_opt;
   if (thread_mask)
   {
-    if (init_master_info(mi,master_info_file,relay_log_info_file, 0))
+    if (init_master_info(mi,master_info_file,relay_log_info_file, 0,
+			 thread_mask))
       slave_errno=ER_MASTER_INFO;
     else if (server_id_supplied && *mi->host)
       slave_errno = start_slave_threads(0 /*no mutex */,
@@ -869,7 +868,8 @@ int change_master(THD* thd, MASTER_INFO* mi)
   thd->proc_info = "Changing master";
   LEX_MASTER_INFO* lex_mi = &thd->lex.mi;
   // TODO: see if needs re-write
-  if (init_master_info(mi, master_info_file, relay_log_info_file, 0))
+  if (init_master_info(mi, master_info_file, relay_log_info_file, 0,
+		       thread_mask))
   {
     send_error(&thd->net, ER_MASTER_INFO);
     unlock_slave_threads(mi);
