@@ -1,7 +1,7 @@
-/* A Bison parser, made from sql_yacc.yy, by GNU bison 1.75.  */
+/* A Bison parser, made by GNU Bison 1.875c.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,9 +23,6 @@
    This special exception was added by the Free Software Foundation
    in version 1.24 of Bison.  */
 
-#ifndef BISON_SQL_YACC_H
-# define BISON_SQL_YACC_H
-
 /* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
@@ -822,9 +819,9 @@
 
 
 
-#ifndef YYSTYPE
+#if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
 #line 44 "sql_yacc.yy"
-typedef union {
+typedef union YYSTYPE {
   int  num;
   ulong ulong_num;
   ulonglong ulonglong_number;
@@ -851,14 +848,15 @@ typedef union {
   enum Item_udftype udf_type;
   thr_lock_type lock_type;
   interval_type interval;
-} yystype;
-/* Line 1281 of /usr/share/bison/yacc.c.  */
-#line 857 "y.tab.h"
-# define YYSTYPE yystype
+} YYSTYPE;
+/* Line 1275 of yacc.c.  */
+#line 854 "sql_yacc.h"
+# define yystype YYSTYPE /* obsolescent; will be withdrawn */
+# define YYSTYPE_IS_DECLARED 1
+# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 
 
 
-#endif /* not BISON_SQL_YACC_H */
 
